#!/usr/bin/python
#central_service.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# This is for services with the central DHN.  Main services are:
#
# 1) register with central service
# 2) list suppliers or customers
# 3) fire a supplier, hire another supplier
# 4) account information and transfers  - done in account.py
# 5) nearnesscheck - meat of function done by nearnesscheck.py
#

import os
import sys
import time
import string

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in central_service.py')

from twisted.internet.defer import Deferred, succeed
from twisted.internet.task import LoopingCall

import lib.misc as misc
import lib.dhnio as dhnio
import lib.settings as settings
import lib.dhnpacket as dhnpacket
import lib.nameurl as nameurl
import lib.packetid as packetid
import lib.commands as commands
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.transport_control as transport_control
import lib.bandwidth as bandwidth
import lib.diskspace as diskspace
import lib.identitycache as identitycache
import lib.automats as automats

import central_connector
import fire_hire
import data_sender

import identitypropagate
import money
import localtester
import backups
import events


_InitDone = False
_Connector = None
_HelloResponseWaiting = False
_HelloCount = 0
_HelloTask = None
_HelloPID = None
_HelloInterval = 60
_InitControlFunc = None
_CentralStatusDict = {}
_LoopSendBandwidthReportsTask = None
_MarketBids = None
_MarketOffers = None

#------------------------------------------------------------------------------ 

OnListSuppliersFunc = None
OnListCustomersFunc = None
OnMarketListFunc = None

#-------------------------------------------------------------------------------

def init(hello_interaval=60):
    global _InitDone
    if _InitDone:
        return
    dhnio.Dprint(4, 'central_service.init')
    transport_control.AddInboxCallback(inbox)
    bandwidth.init()
    _InitDone = True

def shutdown():
    _LoopSendBandwidthReportsTask.cancel()
    return succeed(1)

#------------------------------------------------------------------------------

def inbox(newpacket, proto, host):
    commandhandled = False

    if newpacket.OwnerID not in [ settings.CentralID(), settings.MarketServerID(), ]:
        return False

    if newpacket.Command == commands.Ack():
        Ack(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Receipt():
        Receipt(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.ListContacts():
        ListContacts(newpacket)
        commandhandled = True

    if commandhandled:
        dhnio.Dprint(12, "central_service.inbox [%s] from %s://%s handled" % (newpacket.Command, proto, host))

    return commandhandled

def send2central(command, data, doAck=False, PacketID=None):
    MyID = misc.getLocalID()
    RemoteID = settings.CentralID()
    if PacketID is None:
        PacketID = packetid.UniqueID()
    packet = dhnpacket.dhnpacket(
        command,
        MyID,
        MyID,
        PacketID,
        data,
        RemoteID,)
    transport_control.outbox(packet, doAck)
    #del packet
    return PacketID

def send2market(command, data, doAck=False, PacketID=None):
    MyID = misc.getLocalID()
    RemoteID = settings.MarketServerID()
    if PacketID is None:
        PacketID = packetid.UniqueID()
    packet = dhnpacket.dhnpacket(
        command,
        MyID,
        MyID,
        PacketID,
        data,
        RemoteID,)
    transport_control.outbox(packet, doAck)
    return PacketID

#------------------------------------------------------------------------------

def HelloRequestSuppliers():
    dhnio.Dprint(4, 'central_service.HelloRequestSuppliers')
    SendRequestSuppliers('', True)


#def LoopSendAlive():
#    dhnio.Dprint(4, 'central_service.LoopSendAlive')
#    SendIdentity()
#    reactor.callLater(settings.DefaultAlivePacketTimeOut(), LoopSendAlive)


#-- SENDING --------------------------------------------------------------------

# Register our identity with central
def SendIdentity(doAck=False):
    LocalIdentity = misc.getLocalIdentity()
    dhnio.Dprint(4, "central_service.SendIdentity") #' version=[%s] contacts=[%s]" % (str(LocalIdentity.version), str(LocalIdentity.contacts)))
    data = LocalIdentity.serialize()
    ret = send2central(commands.Identity(), data, doAck, misc.getLocalID())
    return ret

# Say what eccmap we are using for recovery info
# How many suppliers we want (probably same as used by eccmap but just in case)
# Say how much disk we are donating now
def SendSettings(doAck=False, packetID=None):
    if packetID is None:
        packetID = packetid.UniqueID()
    sdict = {}
    sdict['s'] = str(settings.getCentralNumSuppliers())
    # donated = DiskSpace(s=settings.getCentralMegabytesDonated())
    # needed = DiskSpace(s=settings.getCentralMegabytesNeeded())
    # sdict['d'] = str(donated.getValueMb())
    # sdict['n'] = str(needed.getValueMb())
    sdict['d'] = str(diskspace.GetMegaBytesFromString(settings.getCentralMegabytesDonated()))
    sdict['n'] = str(diskspace.GetMegaBytesFromString(settings.getCentralMegabytesNeeded()))
    sdict['e'] = settings.getECC()
    sdict['p'] = str(settings.BasePricePerGBDay())
    sdict['e1'] = str(settings.getEmergencyEmail())
    sdict['e2'] = str(settings.getEmergencyPhone())
    sdict['e3'] = str(settings.getEmergencyFax())
    sdict['e4'] = str(settings.getEmergencyOther()).replace('\n', '<br>')
    sdict['mf'] = settings.getEmergencyFirstMethod()
    sdict['ms'] = settings.getEmergencySecondMethod()
    sdict['ie'] = misc.readExternalIP()
    sdict['il'] = misc.readLocalIP()
    sdict['nm'] = str(settings.uconfig('personal.personal-name'))
    sdict['sn'] = str(settings.uconfig('personal.personal-surname'))
    sdict['nn'] = str(settings.uconfig('personal.personal-nickname'))
    sdict['bt'] = str(settings.uconfig('personal.personal-betatester'))
    i = 0
    for idurl in contacts.getCorrespondentIDs():
        sdict['f%03d' % i] = idurl
        i += 1

    data = dhnio._pack_dict(sdict)
    pid = send2central(commands.Settings(), data, doAck, packetID)
    dhnio.Dprint(4, "central_service.SendSettings PacketID=[%s]" % pid)
    return pid

def SettingsResponse(packet):
    words = packet.Payload.split('\n')
    try:
        status = words[0]
        last_receipt = int(words[1])
    except:
        status = ''
        last_receipt = -1
    dhnio.Dprint(4, "central_service.SettingsResponse: status=[%s] last receipt=[%s]" % (status, str(last_receipt)))

def SendRequestSettings(doAck=False):
    dhnio.Dprint(4, 'central_service.SendRequestSettings')
    return send2central(commands.RequestSettings(), '', doAck)

def SendReplaceSupplier(numORidurl, doAck=False):
    if isinstance(numORidurl, str):
        idurl = numORidurl
    else:
        idurl = contacts.getSupplierID(numORidurl)
    if not idurl:
        dhnio.Dprint(2, "central_service.SendReplaceSupplier ERROR supplier not found")
        return None
    dhnio.Dprint(4, "central_service.SendReplaceSupplier [%s]" % nameurl.GetName(idurl))
    data = 'S\n'+idurl+'\n'+str(contacts.numberForSupplier(idurl))
    ret = send2central(commands.FireContact(), data, doAck)
    events.notify('central_service', 'sent request to dismiss supplier %s' % nameurl.GetName(idurl))
    return ret

def SendChangeSupplier(numORidurl, newidurl, doAck=False):
    if isinstance(numORidurl, str):
        idurl = numORidurl
    else:
        idurl = contacts.getSupplierID(numORidurl)
    if not idurl or not newidurl:
        return None
    dhnio.Dprint(4, "central_service.SendChangeSupplier [%s]->[%s]" % (nameurl.GetName(idurl), nameurl.GetName(newidurl)))
    data = 'N\n'+idurl+'\n'+newidurl
    ret = send2central(commands.FireContact(), data, doAck)

def SendReplaceCustomer(numORidurl, doAck=False):
    if isinstance(numORidurl, str):
        idurl = numORidurl
    else:
        idurl = contacts.getCustomerID(numORidurl)
    if not idurl:
        dhnio.Dprint(2, "central_service.SendReplaceCustomer ERROR customer not found")
        return None
    dhnio.Dprint(4, "central_service.SendReplaceCustomer [%s]" % nameurl.GetName(idurl))
    data = 'C\n'+idurl+'\n'+str(contacts.numberForCustomer(idurl))
    ret = send2central(commands.FireContact(), data, doAck)
    events.notify('central_service', 'sent request to dismiss customer %s' % nameurl.GetName(idurl))
    return ret

def SendRequestSuppliers(data = '', doAck=False):
    dhnio.Dprint(4, "central_service.SendRequestSuppliers")
    return send2central(commands.RequestSuppliers(), data, doAck)

def SendRequestCustomers(data = '', doAck=False):
    dhnio.Dprint(4, "central_service.SendRequestCustomers")
    return send2central(commands.RequestCustomers(), data, doAck)

#------------------------------------------------------------------------------ 
# Account
# User needs to be able to know his balance. We want to keep it on
#   this machine, not check central each time.  Might have a charge
#   for checking with central and keep date of last checkin so user
#   can not check in too often.
#
# Users who have been with DHN for more than 6 months will be able
#    to transfer some of his prepaid/earned balance to another user.
#    This is like a prepaid cellphone transfering balance to another phone.
#
#    A dhnpacket with a commands.Transfer()
#         The data portion says:
#             Amount: 23.4
#             To: http://foo.bar/baz.xml
#
#    With the dhnpacket signed, we know it is legit.  The ID of the destination is there.
#
#
def SendTransfer(DestinationID, Amount, doAck=False):
    dhnio.Dprint(4, "central_service.SendTransfer  DestinationID=%s  Amount=%s" % (DestinationID, str(Amount)))
    data = DestinationID+'\n'+str(Amount)
    return send2central(commands.Transfer(), data, doAck)


def SendRequestReceipt(missing_receipts, doAck=False):
    dhnio.Dprint(4, 'central_service.SendRequestReceipt ' + str(missing_receipts))
    data = string.join(list(missing_receipts), ' ')
    return send2central(commands.RequestReceipt(), data, doAck)

#------------------------------------------------------------------------------ 
# Market
# First we send a Transfer packet to the Central - the destination ID is Market server 
# The packet contains info about bid or offer.
# Central server checks that user have the needed balance and transfer funds to Market server
#
def SendBid(maxamount, price, days, comment, btcaddress, transactionid, doAck=False):
    dhnio.Dprint(4, 'central_service.SendBid ' + str((maxamount, price, days)))
    data = '%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s' % (misc.getLocalID(), 'bid', misc.float2str(maxamount), misc.float2str(price), str(days), str(btcaddress), str(transactionid), str(comment).replace('\n', ''))
    # send bid to the Market server
    return send2market(commands.Transfer(), data, doAck) 

def SendCancelBid(bidID, doAck=False):
    dhnio.Dprint(4, 'central_service.SendCancelBid ' + str(bidID))
    data = '%s\n%s\n%s' % (misc.getLocalID(), 'cancelbid', bidID)
    # send 'cencel bid' to Market server
    return send2market(commands.Transfer(), data, doAck)

def SendOffer(maxamount, minamount, price, days, comment, btcaddress, doAck=False):
    dhnio.Dprint(4, 'central_service.SendOffer ' + str((maxamount, price, days, btcaddress)))
    data = '%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s' % (settings.MarketServerID(), 'offer', misc.float2str(maxamount), misc.float2str(minamount), misc.float2str(price), str(days),  btcaddress, str(comment).replace('\n', ''))
    # send offer to the Central server to check user balance first and than send Receipt to Market server
    return send2central(commands.Transfer(), data, doAck)

def SendCancelOffer(offerID, doAck=False):
    dhnio.Dprint(4, 'central_service.SendCancelOffer ' + str(offerID))
    data = '%s\n%s\n%s' % (misc.getLocalID(), 'canceloffer', offerID)
    # send 'cancel offer' to Market server
    return send2market(commands.Transfer(), data, doAck)

def MarketListAck(newpacket):
    global _MarketBids
    global _MarketOffers
    global OnMarketListFunc
    dhnio.Dprint(4, "central_service.MarketListAck")
    buf = ''
    bidORoffer = None
    _MarketBids = []
    _MarketOffers = []
    for line in newpacket.Payload.splitlines():
        if line.startswith('begin bid'):
            bidORoffer = 'bid'
            continue
        if line.startswith('begin offer'):
            bidORoffer = 'offer'
            continue
        if line.startswith('end'):
            if bidORoffer == 'bid':
                _MarketBids.append(dhnio._unpack_dict(buf))
            elif bidORoffer == 'offer':
                _MarketOffers.append(dhnio._unpack_dict(buf))
            buf = ''
            continue
        buf += line + '\n'
    # dhnio.Dprint(4, '    bids:   ' + str(_MarketBids))
    # dhnio.Dprint(4, '    offers: ' + str(_MarketOffers))
    if OnMarketListFunc is not None:
        OnMarketListFunc()   

def SendRequestMarketList(doAck=True, packetID=None):
    LocalIdentity = misc.getLocalIdentity()
    dhnio.Dprint(4, "central_service.SendRequestMarketList")
    data = LocalIdentity.serialize()
    if packetID is None:
        packetID = packetid.UniqueID()
    # transport_control.RegisterInterest(MarketListAck, settings.MarketServerID(), packetID)
    return send2market(commands.Identity(), data, doAck, packetID)

#--- RECEIVING -----------------------------------------------------------------


def Ack(packet):
    dhnio.Dprint(4, "central_service.Ack packetID=[%s]" % packet.PacketID)
    if packet.CreatorID == settings.MarketServerID():
        MarketListAck(packet)


def Receipt(request):
    dhnio.Dprint(4, "central_service.Receipt " )
    money.InboxReceipt(request)
    if request.OwnerID != settings.MarketServerID():
        transport_control.SendAck(request)
    missing_receipts = money.SearchMissingReceipts()
    if len(missing_receipts) > 0:
        reactor.callLater(5, SendRequestReceipt, missing_receipts)


#S list of suppliers after RequestSuppliers
#s list of suppliers after RequestSuppliers and have troubles (not found)
#fS list of suppliers after FireContact(supplier)
#fs list of suppliers after FireContact(supplier) and have troubles (not found)
#C list of customers after RequestCustomers
#c list of customers after RequestCustomers and have troubles (not found)
#fC list of customers after FireContact(customers)
#fc list of customers after FireContact(customers) and have troubles (not found)
#bS user was banned with negative balance
legal_codes = ['S','C','s','c','fS','fC','fs','fc','bS']
def ListContacts(request):
    global _CentralStatusDict
    global legal_codes
    global OnListSuppliersFunc
    global OnListCustomersFunc

    data = request.Payload
    dhnio.Dprint(6, 'central_service.ListContacts\n%s' % data)
    words = data.split('\n', 1)
    if len(words) < 2:
        dhnio.Dprint(1, 'central_service.ListContacts ERROR wrong data packet [%s]' % str(request.Payload))
        control('ListContactsError', request)
        return

    code = words[0]
    if not code in legal_codes:
        dhnio.Dprint(1, 'central_service.ListContacts ERROR wrong data in the packet [%s] '  % str(request.Payload))
        control('ListContactsError', request)
        return

    current_contacts = []
    clist, tail = dhnio._unpack_list(words[1])

    fire_flag = code.startswith('f')
    ban_flag = code.startswith('b')
    contact_type = code.lower()[-1]
    error_flag = code[-1].islower()

    spaceDict = None
    if tail is not None:
        # extract info about who is alive at the moment
        onlineArray = ''
        spaceDict = dhnio._unpack_dict_from_list(tail)
        if spaceDict.has_key('online'):
            onlineArray = spaceDict.pop('online')
        for i in range(len(onlineArray)):
            if i < len(clist):
                _CentralStatusDict[clist[i]] = onlineArray[i]
        # extract info about contacts local ip
        # if they are in same LAN we need to connect to local IP, not external
        local_ips = {}
        i = 0
        while True:
            idurl_and_local_ip = spaceDict.get('localip%03d' % i, None)
            if idurl_and_local_ip is None:
                break
            try:
                contact_idurl, contact_local_ip = idurl_and_local_ip.split('|')
            except:
                break
            local_ips[contact_idurl] = contact_local_ip
            spaceDict.pop('localip%03d' % i)
            i += 1
            dhnio.Dprint(6, 'central_service.ListContacts got local IP for %s: %s' % (nameurl.GetName(contact_idurl), contact_local_ip))
        identitycache.SetLocalIPs(local_ips)

    #---suppliers
    if contact_type == 's':
        current_contacts = contacts.getSupplierIDs()
        contacts.setSupplierIDs(clist)
        eccmap.init()
        contacts.saveSupplierIDs()
        for supid in contacts.getSupplierIDs():
            if supid.strip() == '':
                error_flag = True
                break
        dhnio.Dprint(4, "central_service.ListContacts (SUPPLIERS) code:%s error:%s length:%d" % (str(code), str(error_flag), len(clist)))
        for oldidurl in current_contacts:
            if oldidurl:
                if oldidurl not in clist:
                    events.info('central_service', 'supplier %s were disconnected' % nameurl.GetName(oldidurl),)
                    misc.writeSupplierData(oldidurl, 'disconnected', time.strftime('%d%m%y %H:%M:%S'))
                    dhnio.Dprint(6, 'central_service.ListContacts supplier %s were disconnected' % nameurl.GetName(oldidurl))
        for newidurl in clist:
            if newidurl:
                if newidurl not in current_contacts:
                    transport_control.ClearAliveTime(newidurl)
                    misc.writeSupplierData(newidurl, 'connected', time.strftime('%d%m%y %H:%M:%S'))
                    events.info('central_service', 'new supplier %s connected' % nameurl.GetName(newidurl), '',)
                    dhnio.Dprint(6, 'central_service.ListContacts new supplier %s connected' % nameurl.GetName(newidurl))
        backups.SetSupplierList(clist)
        if not fire_flag and current_contacts != clist:
            identitypropagate.suppliers(wide=True)
            #for newidurl in clist:
            #    identitycache.Remove(newidurl)
            #identitypropagate.RealSendSuppliers()
        if OnListSuppliersFunc is not None:
            OnListSuppliersFunc()


    #---customers
    elif contact_type == 'c':
        current_contacts = contacts.getCustomerIDs()
        contacts.setCustomerIDs(clist)
        contacts.saveCustomerIDs()
        if spaceDict is not None:
            dhnio._write_dict(settings.CustomersSpaceFile(), spaceDict)
            reactor.callLater(3, localtester.TestUpdateCustomers)
        if not fire_flag and current_contacts != clist:
            identitypropagate.customers(wide=True)
        dhnio.Dprint(4, "central_service.ListContacts (CUSTOMERS) code:%s error:%s length:%d" % (str(code), str(error_flag), len(clist)))
        for oldidurl in current_contacts:
            if oldidurl not in clist:
                events.info('central_service', 'customer %s were disconnected' % nameurl.GetName(oldidurl),)
                dhnio.Dprint(6, 'central_service.ListContacts customer %s were disconnected' % nameurl.GetName(oldidurl))
        for newidurl in clist:
            if newidurl not in current_contacts:
                transport_control.ClearAliveTime(newidurl)
                events.info('central_service', 'new customer %s connected' % nameurl.GetName(newidurl))
                dhnio.Dprint(6, 'central_service.ListContacts new customer %s connected' % nameurl.GetName(newidurl))
        if OnListCustomersFunc is not None:
            OnListCustomersFunc()

    #---fire_flag
    if fire_flag:
        if contact_type == 's':
            index = -1
            for index in range(len(clist)):
                if clist[index] != current_contacts[index]:
                    break
            if index >= 0:
                # we want to send our Identity to new supplier
                # and than ask a list of files he have
                # so it should start rebuilding backups immediately
                # right after we got ListFiles from him
                identitypropagate.single(clist[index], wide=True) 

    #---ban_flag
    if ban_flag:
        events.notify('central_service', 'you have negative balance, all your suppliers was removed', '',)
        dhnio.Dprint(2, 'central_service.ListContacts !!! you have negative balance, all your suppliers was removed !!!')

    #---error_flag
    if error_flag:
        #reactor.callLater(settings.DefaultNeedSuppliersPacketTimeOut(), SendRequestSuppliers)
        events.info('central_service', 'could not find available suppliers',
                     'Central server can not find available suppliers for you.\nCheck your central settings.\n',)
        dhnio.Dprint(2, 'central_service.ListContacts !!! could not find available suppliers !!!')

    #identitypropagate.FetchNecesaryContacts()

    #---send ack            
    transport_control.SendAck(request)

    #---automats
    if contact_type == 's':
        central_connector.A('list-suppliers', clist)
        fire_hire.A('list-suppliers', (current_contacts, clist))
        data_sender.A('restart')
    elif contact_type == 'c':
        central_connector.A('list-customers', clist)
        
    #---transport_udp
    if transport_control._TransportUDPEnable:
        import lib.transport_udp as transport_udp
        new_contacts = contacts.getContactsAndCorrespondents()
        transport_udp.ListContactsCallback(current_contacts, new_contacts)



#--- NEARNESS ------------------------------------------------------------------

#  NearnessCheck request coming from central
#  When request comes in we start the check with callback
#  going to NearnessResult which will send results to central.
def NearnessCheck(request):
    dhnio.Dprint(4, "central_service.NearnessCheck")

def NearnessResult():
    dhnio.Dprint(4, "central_service.NearnessResult")

#--- BANDWIDTH -----------------------------------------------------------------

def LoopSendBandwidthReport():
    global _LoopSendBandwidthReportsTask
    if _LoopSendBandwidthReportsTask is not None:
        return
    pid = SendBandwidthReport()
    interval = settings.DefaultBandwidthReportTimeOut()
    if pid is not None:
        transport_control.RegisterInterest(ReceiveBandwidthAck, settings.CentralID(), pid)
    else:
        interval /= 10.0
    _LoopSendBandwidthReportsTask = reactor.callLater(interval, LoopSendBandwidthReport)

def SendBandwidthReport():
    listin, listout = bandwidth.files2send()
    if len(listin) == 0 and len(listout) == 0:
        dhnio.Dprint(4, 'central_service.SendBandwidthReport skip')
        return None
    dhnio.Dprint(4, 'central_service.SendBandwidthReport')
    src = ''
    for filepath in listin:
        filename = os.path.basename(filepath)
        if len(filename) != 6:
            dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING incorrect filename ' + filepath)
            continue
        s = dhnio.ReadBinaryFile(filepath)
        if not s:
            # dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING %s is empty' % filepath)
            continue
        src += '[in] %s\n' % filename
        src += s.strip()
        src += '\n[end]\n'
    for filepath in listout:
        filename = os.path.basename(filepath)
        if len(filename) != 6:
            dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING incorrect filename ' + filepath)
            continue
        s = dhnio.ReadBinaryFile(filepath)
        if not s:
            # dhnio.Dprint(6, 'central_service.SendBandwidthReport WARNING %s is empty' % filepath)
            continue
        src += '[out] %s\n' % filename
        src += s.strip()
        src += '\n[end]\n'
    # dhnio.Dprint(4, '\n' + src)
    if src.strip() == '':
        # dhnio.Dprint(4, 'central_service.SendBandwidthReport WARNING src is empty. skip.')
        return None
    return send2central(commands.BandwidthReport(), src)

def ReceiveBandwidthAck(packet):
    dhnio.Dprint(4, 'central_service.ReceiveBandwidthAck')
    for line in packet.Payload.split('\n'):
        try:
            typ, filename = line.strip().split(' ')
        except:
            continue
        if typ == '[in]':
            filepath = os.path.join(settings.BandwidthInDir(), filename)
        elif typ == '[out]':
            filepath = os.path.join(settings.BandwidthOutDir(), filename)
        else:
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck ERROR typ=%s filename=%s' % (typ, filename))
            continue
        if not os.path.isfile(filepath):
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck ERROR %s not found' % filepath)
            continue
        if os.path.isfile(filepath+'.sent'):
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck WARNING %s already sent' % filepath)
            continue
        try:
            os.rename(filepath, filepath+'.sent')
        except:
            dhnio.Dprint(2, 'central_service.ReceiveBandwidthAck ERROR can not rename %s' % filepath)
            dhnio.DprintException()
        dhnio.Dprint(4, '  %s %s accepted' % (typ, filename))


#-------------------------------------------------------------------------------


def main():
    try:
        from twisted.internet import reactor
    except:
        sys.exit('Error initializing twisted.internet.reactor in central_service.py')

    transport_control.init()
    SendIdentity()
    reactor.run()


if __name__ == '__main__':
    main()






# -*- coding: ascii -*-
#
#  Util/asn1.py : Minimal support for ASN.1 DER binary encoding.
#
# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# No rights are reserved.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ===================================================================

from Crypto.Util.number import long_to_bytes, bytes_to_long

__all__ = [ 'DerObject', 'DerInteger', 'DerSequence' ]

class DerObject:
	typeTags = { 'SEQUENCE':'\x30', 'BIT STRING':'\x03', 'INTEGER':'\x02' }

	def __init__(self, ASN1Type=None):
		self.typeTag = self.typeTags.get(ASN1Type, ASN1Type)
		self.payload = ''

	def _lengthOctets(self, payloadLen):
		'''
		Return an octet string that is suitable for the BER/DER
		length element if the relevant payload is of the given
		size (in bytes).
		'''
		if payloadLen>127:
			encoding = long_to_bytes(payloadLen)
			return chr(len(encoding)+128) + encoding
		return chr(payloadLen)

	def encode(self):
		return self.typeTag + self._lengthOctets(len(self.payload)) + self.payload	

	def _decodeLen(self, idx, str):
		'''
		Given a string and an index to a DER LV,
		this function returns a tuple with the length of V
		and an index to the first byte of it.
		'''
		length = ord(str[idx])
		if length<=127:
			return (length,idx+1)
		else:
			payloadLength = bytes_to_long(str[idx+1:idx+1+(length & 0x7F)])
			if payloadLength<=127:
				raise ValueError("Not a DER length tag.")
			return (payloadLength, idx+1+(length & 0x7F))

	def decode(self, input, noLeftOvers=0):
		try:
			self.typeTag = input[0]
			if (ord(self.typeTag) & 0x1F)==0x1F:
				raise ValueError("Unsupported DER tag")
			(length,idx) = self._decodeLen(1,input)
			if noLeftOvers and len(input) != (idx+length):
				raise ValueError("Not a DER structure")
			self.payload = input[idx:idx+length]
		except IndexError:
			raise ValueError("Not a valid DER SEQUENCE.")
		return idx+length

class DerInteger(DerObject):
	def __init__(self, value = 0):
		DerObject.__init__(self, 'INTEGER')
		self.value = value

	def encode(self):
		self.payload = long_to_bytes(self.value)
		if ord(self.payload[0])>127:
			self.payload = '\x00' + self.payload
		return DerObject.encode(self)

	def decode(self, input, noLeftOvers=0):
		tlvLength = DerObject.decode(self, input,noLeftOvers)
		if ord(self.payload[0])>127:
			raise ValueError ("Negative INTEGER.")
		self.value = bytes_to_long(self.payload)
		return tlvLength
				
class DerSequence(DerObject):
	def __init__(self):
		DerObject.__init__(self, 'SEQUENCE')
		self._seq = []
	def __delitem__(self, n):
		del self._seq[n]
	def __getitem__(self, n):
		return self._seq[n]
	def __setitem__(self, key, value):
		self._seq[key] = value	
	def __setslice__(self,i,j,sequence):
		self._seq[i:j] = sequence
	def __delslice__(self,i,j):
		del self._seq[i:j]
	def __getslice__(self, i, j):
		return self._seq[max(0, i):max(0, j)]
	def __len__(self):
		return len(self._seq)
	def append(self, item):
		return self._seq.append(item)

	def hasOnlyInts(self):
		if not self._seq: return 0
		test = 0
		for item in self._seq:
			try:
				test += item
			except TypeError:
				return 0
		return 1

	def encode(self):
		'''
		Return the DER encoding for the ASN.1 SEQUENCE containing
		the non-negative integers and longs added to this object.
		'''
		self.payload = ''
		for item in self._seq:
			try:
				self.payload += item
			except:
				try:
					self.payload += DerInteger(item).encode()
				except:
					raise ValueError("Trying to DER encode an unknown object")
		return DerObject.encode(self)

	def decode(self, input,noLeftOvers=0):
		'''
		This function decodes the given string into a sequence of
		ASN.1 objects. Yet, we only know about unsigned INTEGERs.
		Any other type is stored as its rough TLV. In the latter
		case, the correctectness of the TLV is not checked.
		'''
		self._seq = []
		try:
			tlvLength = DerObject.decode(self, input,noLeftOvers)
			if self.typeTag!=self.typeTags['SEQUENCE']:
				raise ValueError("Not a DER SEQUENCE.")
			# Scan one TLV at once
			idx = 0
			while idx<len(self.payload):
				typeTag = self.payload[idx]
				if typeTag==self.typeTags['INTEGER']:
					newInteger = DerInteger()
					idx += newInteger.decode(self.payload[idx:])
					self._seq.append(newInteger.value)
				else:
					itemLen,itemIdx = self._decodeLen(idx+1,self.payload)
					self._seq.append(self.payload[idx:itemIdx+itemLen])
					idx = itemIdx + itemLen
		except IndexError:
			raise ValueError("Not a valid DER SEQUENCE.")
		return tlvLength




#!/usr/bin/python
#summary.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#


import os
import sys
import time


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in summary.py')


import lib.settings as settings
import lib.misc as misc
import lib.diskspace as diskspace
import lib.transport_control as transport_control
import lib.contacts as contacts
import lib.dhnio as dhnio


import backup_db


class _SummaryInfo:
    def __init__(self, guiSummaryCall=None):
        self._guiSummaryCall = guiSummaryCall
        self.identity = misc.getIDName() # going to leave in for people running from command line
        self.balance = ''
        self.burnRate = ''
        self.ds = diskspace.DiskSpace()

    def GetActiveCount(self):
        activeCount = 0
        for idurl in contacts.getSupplierIDs():
            if idurl:
                if transport_control.ContactIsAlive(idurl):
                    activeCount += 1
        return activeCount

    def SetGUISummaryCall(self, guiSummaryCall):
        dhnio.Dprint(14, 'summary.SetGUISummaryCall')
        self._guiSummaryCall = guiSummaryCall

    def UpdateStatusInfo(self, doPoll = False):
        try:
            supplierStatus = str(self.GetActiveCount()) + "/" + str(settings.getCentralNumSuppliers()) + " online"
            spaceStatus = self.ds.getValueBest(backup_db.GetTotalBackupsSize() * 2) + "/" + settings.getCentralMegabytesNeeded()
            daysAtBurnRate = "pos cash flow"
            balanceStatus = ''
            if self.balance != '' and self.burnRate != '':
                if self.burnRate < 0.0:
                    daysAtBurnRate = self.balance / (-self.burnRate)
                    if daysAtBurnRate > 365:
                        daysAtBurnRate = str(int(daysAtBurnRate/365)) + " years"
                    elif daysAtBurnRate > 30:
                        daysAtBurnRate = str(int(daysAtBurnRate/30)) + " months"
                    else:
                        daysAtBurnRate = str(int(daysAtBurnRate)) + " days"
                balanceStatus = '$' + ("%.2f" % self.balance) + " (" + daysAtBurnRate + ")"

            if self._guiSummaryCall is not None:
                dhnio.Dprint(14, 'summary.UpdateStatusInfo about to do guisummarycall')
                self._guiSummaryCall(supplierStatus, spaceStatus, balanceStatus)
            else:
                dhnio.Dprint(14, "Identity:  " + self.identity)
                dhnio.Dprint(14, "Suppliers: " + supplierStatus)
                dhnio.Dprint(14, "Space:     " + spaceStatus)
                dhnio.Dprint(14, "Balance:   " + balanceStatus)
        except:
            dhnio.DprintException()

        if doPoll:
            reactor.callLater(900, self.UpdateStatusInfo, doPoll)
        #things that can change the summary, change in space or suppliers (on save in settings), change in space used, online suppliers ...

    def SetBalanceAndBurnRate(self, balance, burnRate):
        dhnio.Dprint(14, 'summary.SetBalanceAndBurnRate ' + str(balance) + ", " + str(burnRate))
        self.balance = balance
        self.burnRate = burnRate
        self.UpdateStatusInfo()


# Veselin, please ask me before making changes to this, summary.init gets called before anything calling calling summary
SetBalanceAndBurnRate = None
SetGUISummaryCall = None
UpdateStatusInfo = None

def init(callback=None):
    dhnio.Dprint(4, 'summary.init')
    global SetBalanceAndBurnRate
    global SetGUISummaryCall
    global UpdateStatusInfo

    _Summary = _SummaryInfo(callback)

    SetBalanceAndBurnRate = _Summary.SetBalanceAndBurnRate
    SetGUISummaryCall = _Summary.SetGUISummaryCall
    UpdateStatusInfo = _Summary.UpdateStatusInfo

    UpdateStatusInfo(True) # start off the one loop
    
    
    


#!/usr/bin/python
#transport_tcp.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#


import os
import sys
import time

if __name__ == '__main__':
    sys.path.insert(0, os.path.abspath('..'))


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in transport_tcp.py')


from twisted.internet import task
from twisted.internet import protocol
#from twisted.internet.protocol import ServerFactory, ClientFactory
from twisted.protocols import basic
from twisted.internet.defer import Deferred, fail 
from twisted.python import log


import dhnio
import misc
import tmpfile

_SendStatusFunc = None
_ReceiveStatusFunc = None
_SendControlFunc = None
_ReceiveControlFunc = None
_RegisterTransferFunc = None
_UnRegisterTransferFunc = None
_ByTransferID = {}

#------------------------------------------------------------------------------

def init(   sendStatusFunc=None, 
            receiveStatusFunc=None, 
            sendControl=None, 
            receiveControl=None,
            registerTransferFunc=None,
            unregisterTransferFunc=None):
    dhnio.Dprint(4, 'transport_tcp.init')
    global _SendStatusFunc
    global _ReceiveStatusFunc
    global _RegisterTransferFunc
    global _UnRegisterTransferFunc
    if sendStatusFunc is not None:
        _SendStatusFunc = sendStatusFunc
    if receiveStatusFunc is not None:
        _ReceiveStatusFunc = receiveStatusFunc
    if sendControl is not None:
        _SendControlFunc = sendControl 
    if registerTransferFunc is not None:
        _RegisterTransferFunc = registerTransferFunc
    if unregisterTransferFunc is not None:
        _UnRegisterTransferFunc = unregisterTransferFunc
    

def send(filename, host, port, do_status_report=True, send_control_func=None, result_defer=None, description=''):
    global _SendControlFunc
    sender = MySendingFactory(
        filename, host, int(port), 
        result_defer=result_defer, 
        do_status_report=do_status_report, 
        send_control_func=(send_control_func or _SendControlFunc),
        description=description)
    reactor.connectTCP(host, int(port), sender)


def cancel(transferID):
    global _ByTransferID
    inst = _ByTransferID.get(transferID, None)
    if not inst:
        dhnio.Dprint(2, "transport_tcp.cancel ERROR %s is not found" % str(transferID))
        return False
    if isinstance(inst, MySendingProtocol):
        inst.fileSender.stopProducing()
        dhnio.Dprint(8, "transport_tcp.cancel sending %s" % str(transferID))
        return True
    if isinstance(inst, MyReceiveProtocol):
        reactor.callLater(0, inst.transport.loseConnection)
        dhnio.Dprint(8, "transport_tcp.cancel receiving %s" % str(transferID))
        return True
    dhnio.Dprint(2, "transport_tcp.cancel ERROR incorrect instance for %s" % str(transferID))
    return False
        

def receive(port, receive_control_func=None):
    global _ReceiveControlFunc
    dhnio.Dprint(8, "transport_tcp.receive going to listen on port "+ str(port))

    def _try_receiving(port, count, receive_control_func):
        dhnio.Dprint(10, "transport_tcp.receive count=%d" % count)
        f = MyReceiveFactory(receive_control_func)
        try:
            mylistener = reactor.listenTCP(int(port), f)
        except:
            mylistener = None
            dhnio.DprintException()
        return mylistener

    def _loop(port, result, count, receive_control_func):
        if count > 3:
            dhnio.Dprint(1, "transport_tcp.receive WARNING port %s is busy!" % str(port))
            result.errback(None)
            return
        l = _try_receiving(port, count, receive_control_func)
        if l is not None:
            dhnio.Dprint(8, "transport_tcp.receive started on port "+ str(port))
            result.callback(l)
            return
        reactor.callLater(1, _loop, port, result, count+1, receive_control_func)

    res = Deferred()
    _loop(port, res, 0, receive_control_func or _ReceiveControlFunc)
    return res

#------------------------------------------------------------------------------

class MyFileSender(basic.FileSender):
    
    def __init__(self, protocol):
        self.protocol = protocol
        self.bytesLastRead = 0
    
    def resumeProducing(self):
        #sys.stdout.write('=')
        chunk = ''
        if self.file:
            more_bytes = self.CHUNK_SIZE
            if self.protocol.factory.send_control_func:
                more_bytes = self.protocol.factory.send_control_func(self.bytesLastRead, self.CHUNK_SIZE)
            try:
                chunk = self.file.read(more_bytes)
            except:
                chunk = ''
            self.bytesLastRead = len(chunk)
        # print len(chunk)
        if not chunk:
            self.file = None
            self.bytesLastRead = 0
            self.consumer.unregisterProducer()
            if self.deferred:
                self.deferred.callback(self.lastSent)
                self.deferred = None
            return
        if self.transform:
            chunk = self.transform(chunk)
        self.consumer.write(chunk)
        self.lastSent = chunk[-1]


# In Putter "self.factory" references the parent object, so we can
# access arguments like "host", "port", and "filename"
class MySendingProtocol(protocol.Protocol):
    fin = None
    sentBytes = 0
    transfer_id = None
    
    def getSentBytes(self):
        # print 'getSentBytes, TCP', self.transfer_id, self.sentBytes
        return self.sentBytes
    
    def connectionMade(self):
        global _SendStatusFunc
        global _RegisterTransferFunc
        global _ByTransferID
        if not os.path.isfile(self.factory.filename):
            # dhnio.Dprint(6, 'transport_tcp.MySendingProtocol.connectionMade WARNING file %s was not found but we get connected to the %s' % (self.factory.filename, self.factory.host))
            if self.factory.do_status_report:
                _SendStatusFunc(self.transport.getPeer(), self.factory.filename,
                    'failed', 'tcp', None, 'file was not found')
            self.transport.loseConnection()
            if self.factory.result_defer is not None:
                if not self.factory.result_defer.called:
                    self.factory.result_defer.errback('failed')
                    self.factory.result_defer = None
            return
        try:
            sz = os.path.getsize(self.factory.filename)
            self.fin = open(self.factory.filename, 'rb')
        except:
            dhnio.DprintException()
            if self.factory.do_status_report:
                _SendStatusFunc(self.transport.getPeer(), self.factory.filename,
                    'failed', 'tcp', None, 'error opening file')
            self.transport.loseConnection()
            if self.factory.result_defer is not None:
                if not self.factory.result_defer.called:
                    self.factory.result_defer.errback('failed')
                    self.factory.result_defer = None
            return

        self.fileSender = MyFileSender(self)
        d = self.fileSender.beginFileTransfer(self.fin, self.transport, self.transformData)
        d.addCallback(self.finishedTransfer)
        d.addErrback(self.transferFailed)
        self.transfer_id = _RegisterTransferFunc(
            'send', (self.transport.getPeer().host, int(self.transport.getPeer().port)), 
            self.getSentBytes, self.factory.filename, sz, self.factory.description)
        _ByTransferID[self.transfer_id] = self

    def transformData(self, data):
        self.sentBytes += len(data)
        return data

    def finishedTransfer(self, result):
        global _SendStatusFunc
        self.transport.loseConnection()
        try:
            self.fin.close()
        except:
            dhnio.Dprint(1, 'transport_tcp.MySendingProtocol.finishedTransfer ERROR close file failed')
        if self.factory.do_status_report:
            _SendStatusFunc(self.transport.getPeer(), self.factory.filename, 'finished', 'tcp')
        if self.factory.result_defer is not None:
            if not self.factory.result_defer.called:
                self.factory.result_defer.callback('finished')
                self.factory.result_defer = None

    def transferFailed(self, err):
        global _SendStatusFunc
        dhnio.Dprint(6, 'transport_tcp.MySendingProtocol.transferFailed NETERROR host=%s file=%s error=%s' % (str(self.factory.host), str(self.factory.filename), str(err.getErrorMessage())))
        self.transport.loseConnection()
        try:
            self.fin.close()
        except:
            dhnio.Dprint (1, 'transport_tcp.MySendingProtocol.transferFailed ERROR closing file %s' % self.factory.filename)
        if self.factory.do_status_report:
            _SendStatusFunc(self.transport.getPeer(), self.factory.filename, 'failed', 'tcp', err, 'transfer failed')
        if self.factory.result_defer is not None:
            if not self.factory.result_defer.called:
                self.factory.result_defer.errback('failed')
                self.factory.result_defer = None

    def connectionLost(self, reason):
        global _UnRegisterTransferFunc
        global _ByTransferID
        if self.transfer_id is not None:
            _UnRegisterTransferFunc(self.transfer_id)
            _ByTransferID.pop(self.transfer_id)
        # dhnio.Dprint(6, 'transport_tcp.MySendingProtocol.connectionLost with %s' % str(self.transport.getPeer()))
        self.transport.loseConnection()
        try:
            self.fin.close()
        except:
            pass

class MySendingFactory(protocol.ClientFactory):
    def __init__(self, filename, host, port, result_defer=None, do_status_report=True, send_control_func=None, description=''):
        self.filename = filename
        self.host = host
        self.port = port
        self.protocol = MySendingProtocol
        self.result_defer = result_defer
        self.do_status_report = do_status_report
        self.send_control_func = send_control_func # callback which reads from the file
        self.description = description 

    def clientConnectionFailed(self, connector, reason):
        global _SendStatusFunc
        protocol.ClientFactory.clientConnectionFailed(self, connector, reason)
        if self.do_status_report:
            _SendStatusFunc(connector.getDestination(),
                self.filename, 'failed', 'tcp', reason, 'connection failed')
        if self.result_defer is not None:
            if not self.result_defer.called:
                self.result_defer.errback('failed')
                self.result_defer = None
        name = str(reason.type.__name__)
        dhnio.Dprint(12, 'transport_tcp.clientConnectionFailed NETERROR [%s] with %s:%s' % (
            name,
            connector.getDestination().host,
            connector.getDestination().port,))

#------------------------------------------------------------------------------

class MyReceiveProtocol(protocol.Protocol):
    def getReceivedBytes(self):
        return self.receivedBytes
    
    def connectionMade(self):
        global _RegisterTransferFunc
        global _ByTransferID
        self.filename = None     # string with path/filename
        self.fd = None           # integer file descriptor like os.open() returns
        self.peer = None
        self.receivedBytes = 0
        self.transfer_id = None
        self.blocked = False
        self.resume_receiving_task = None
        if self.peer is None:
            self.peer = self.transport.getPeer()
        else:
            if self.peer != self.transport.getPeer():
                raise Exception("transport_tcp.MyReceiveProtocol.connectionMade NETERROR thought we had one object per connection")
        try:
            from transport_control import black_IPs_dict
            if self.peer.host in black_IPs_dict().keys() or '.'.join(self.peer.host.split('.')[:-1]) in black_IPs_dict().keys():
                # dhnio.Dprint(12, 'transport_tcp.MyReceiveProtocol.connectionMade from BLACK IP: %s, %s' % (str(self.peer.host), str(black_IPs_dict()[self.peer.host])) )
                self.blocked = True
                self.transport.loseConnection()
                return
        except:
            pass
        if self.filename is None:
            self.fd, self.filename = tmpfile.make("tcp-in")
        else:   
            raise Exception("transport_tcp.MyReceiveProtocol.connectionMade has second connection in same object")
        if self.fd is None:
            raise Exception("transport_tcp.MyReceiveProtocol.connectionMade error opening temporary file")
        self.transfer_id = _RegisterTransferFunc(
            'receive', (self.transport.getPeer().host, int(self.transport.getPeer().port)), 
            self.getReceivedBytes, self.filename, -1)
        _ByTransferID[self.transfer_id] = self

    def dataReceived(self, data):
        if self.blocked:
            return
        if self.fd is None:
            raise Exception('transport_tcp.MyReceiveProtocol.dataReceived from %s but file was not opened' % str(self.peer))
        amount = len(data)
        os.write(self.fd, data)
        self.receivedBytes += amount
        if self.factory.receive_control_func is not None:
            seconds_pause = self.factory.receive_control_func(len(data))
            if seconds_pause > 0:
                self.transport.pauseProducing()
                self.resume_receiving_task = reactor.callLater(seconds_pause, self.resumeReceiving)

    def resumeReceiving(self):
        self.resume_receiving_task = None
        self.transport.resumeProducing()

    def connectionLost(self, reason):
        global _ReceiveStatusFunc
        global _UnRegisterTransferFunc
        global _ByTransferID
        if self.resume_receiving_task:
            self.resume_receiving_task.cancel()
        if self.transfer_id is not None:
            _UnRegisterTransferFunc(self.transfer_id)
            _ByTransferID.pop(self.transfer_id)
        # dhnio.Dprint(6, 'transport_tcp.MyReceiveProtocol.connectionLost with %s' % str(self.transport.getPeer()))
        if self.fd is not None:
            try:
                os.close(self.fd)
            except:
                dhnio.DprintException()
        if self.filename is not None:
            _ReceiveStatusFunc(self.filename, "finished", 'tcp', self.transport.getPeer(), reason)


class MyReceiveFactory(protocol.ServerFactory):
    def __init__(self, receive_control_func):
        self.receive_control_func = receive_control_func
        
    def buildProtocol(self, addr):
        p = MyReceiveProtocol()
        p.factory = self
        return p

#------------------------------------------------------------------------------

def SendStatusFuncDefault(host, filename, status, proto='', error=None, message=''):
    try:
        from transport_control import sendStatusReport
        sendStatusReport(host, filename, status, proto, error, message)
    except:
        dhnio.DprintException()

def ReceiveStatusFuncDefault(filename, status, proto='', host=None, error=None, message=''):
    try:
        from transport_control import receiveStatusReport
        receiveStatusReport(filename, status, proto, host, error, message)
    except:
        dhnio.DprintException()

def SendControlFuncDefault(prev_read, chunk_size):
    return chunk_size

def ReceiveControlFuncDefault(new_data_size):
    return 0

def RegisterTransferFuncDefault(send_or_receive, remote_address, callback, filename, size, description=''):
    try:
        from transport_control import register_transfer
        return register_transfer('tcp', send_or_receive, remote_address, callback, filename, size, description)
    except:
        dhnio.DprintException()
        return None
    
def UnRegisterTransferFuncDefault(transfer_id):
    try:
        from transport_control import unregister_transfer
        return unregister_transfer(transfer_id)
    except:
        dhnio.DprintException()
    return None

_SendStatusFunc = SendStatusFuncDefault
_ReceiveStatusFunc = ReceiveStatusFuncDefault
_SendControlFunc = SendControlFuncDefault
_ReceiveControlFunc = ReceiveControlFuncDefault
_RegisterTransferFunc = RegisterTransferFuncDefault
_UnRegisterTransferFunc = UnRegisterTransferFuncDefault


#-------------------------------------------------------------------------------

def usage():
        print '''
args:
transport_tcp.py [port]                                   -  to receive
transport_tcp.py [host] [port] [file name]                -  to send a file
transport_tcp.py [host] [port] [file name] [interval]     -  to start sending continously
'''



def mytest():
    dhnio.SetDebug(18)

    filename = "transport_tcp.pyc"              # just some file to send
##    host = "work.offshore.ai"
##    host = "89.223.30.208"
    host = 'localhost'
    port = 7771

    if len(sys.argv) == 4:
        send(sys.argv[3], sys.argv[1], sys.argv[2])
    elif len(sys.argv) == 2:
        r = receive(sys.argv[1])
        print r
    else:
        usage()
        sys.exit()
    reactor.run()

bytes_counter = 0
time_counter = time.time()
count_period = 1
current_chunk = 1

def _my_receive_control(new_data_size):
    try:
        from transport_control import ReceiveTrafficControl
        return ReceiveTrafficControl(new_data_size)
    except:
        dhnio.DprintException()
        return 0
    
def _my_send_control(prev_read_size, chunk_size):
    try:
        from transport_control import SendTrafficControl
        return SendTrafficControl(prev_read_size, chunk_size)
    except:
        dhnio.DprintException()
        return chunk_size

#def _my_monitor():
#    import transport_control
#    src = ''
#    for address in transport_control._LiveTransfers.keys():
#        for obj in transport_control._LiveTransfers[address]['send']:
#            src += '%d to %s, ' % (obj.sentBytes, str(address))
#        for obj in transport_control._LiveTransfers[address]['receive']:
#            src += '%d from %s, ' % (obj.receivedBytes, str(address))
#    print '>>>', src
#    reactor.callLater(1, _my_monitor)

if __name__ == "__main__":
    #import datahaven.p2p.memdebug as memdebug
    #memdebug.start(8080)
    dhnio.init()
    dhnio.SetDebug(20)
    dhnio.LifeBegins()

    from twisted.internet.defer import setDebugging
    setDebugging(True)

    # reactor.callLater(1, _my_monitor)

    # def _test_cancel():
    #     import transport_control as tc
    #     for t in tc.current_transfers():
    #         cancel(t.transfer_id)

    if len(sys.argv) == 2:
        r = receive(sys.argv[1], receive_control_func=_my_receive_control)
        # reactor.callLater(5, _test_cancel)
        reactor.run()
    elif len(sys.argv) == 4:
        _SendStatusFunc = lambda host, filename, status, proto='', error=None, message='': sys.stdout.write('%s to %s is %s\n' % (filename, host, status))
        send(sys.argv[3], sys.argv[1], sys.argv[2], send_control_func=_my_send_control) # .addBoth(lambda x: dhnio.Dprint(2, 'RESULT:%s'%str(x)))
        # reactor.callLater(1, _test_cancel)
        reactor.run()
    elif len(sys.argv) == 5:
        _SendStatusFunc = lambda host, filename, status, proto='', error=None, message='': sys.stdout.write('%s to %s is %s\n' % (filename, host, status))
        l = task.LoopingCall(send, sys.argv[3], sys.argv[1], sys.argv[2])
        l.start(float(sys.argv[4]))
        reactor.run()
    else:
        usage()




#!/usr/bin/python
#dhnicon.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

USE_TRAY_ICON = True
LABEL = 'DataHaven.NET'
    
_IconObject = None
_ControlFunc = None

_IconsDict = {
    'red':      'icon-red.png',
    'green':    'icon-green.png',
    'gray':     'icon-gray.png',
    'yellow':   'icon-yellow.png',
}

#------------------------------------------------------------------------------ 

def init(icons_path, icons_dict):
    global _IconObject
    global USE_TRAY_ICON
    if not USE_TRAY_ICON:
        return

    import wx
    
    from twisted.internet import reactor


    def create_menu_item(menu, label, func, icon=None):
        item = wx.MenuItem(menu, -1, label)
        menu.Bind(wx.EVT_MENU, func, id=item.GetId())
        if icon is not None:
            item.SetBitmap(wx.Bitmap(icon))
        menu.AppendItem(item)
        return item
    
    def icons_dict():
        global _IconsDict
        return _IconsDict
    
    class MyTaskBarIcon(wx.TaskBarIcon):
        def __init__(self, icons_path, current_icon_name=None):
            super(MyTaskBarIcon, self).__init__()
            self.icons_path = icons_path
            self.icons = {}
            for name, filename in icons_dict().items():
                self.icons[name] = wx.IconFromBitmap(wx.Bitmap(os.path.join(icons_path, filename)))
            if len(self.icons) == 0:
                self.icons['default'] = ''
            if current_icon_name is not None and current_icon_name in self.icons.keys():
                self.current = current_icon_name
            else:                
                self.current = self.icons.keys()[0]
            self.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.on_left_down)
            self.select_icon(self.current)

        def items_dict(self):
            return {
                'show':     os.path.join(self.icons_path, 'expand24x24.png'),
                'hide':     os.path.join(self.icons_path, 'collapse24x24.png'),
                'toolbar':  os.path.join(self.icons_path, 'tools24x24.png'),
                'restart':  os.path.join(self.icons_path, 'restart24x24.png'),
                'reconnect':os.path.join(self.icons_path, 'network24x24.png'),
                'shutdown': os.path.join(self.icons_path, 'shutdown24x24.png'),}
        
        def CreatePopupMenu(self):
            menu = wx.Menu()
            icons = self.items_dict()
            create_menu_item(menu, 'show', self.on_show, icons.get('show', None))
            create_menu_item(menu, 'hide', self.on_hide, icons.get('hide', None))
            create_menu_item(menu, 'toolbar', self.on_toolbar, icons.get('toolbar', None))
            menu.AppendSeparator()
            create_menu_item(menu, 'reconnect', self.on_reconnect, icons.get('reconnect', None))
            create_menu_item(menu, 'restart', self.on_restart, icons.get('restart', None))
            create_menu_item(menu, 'shutdown', self.on_exit, icons.get('shutdown', None))
            self.menu = menu
            return menu

        def on_left_down(self, event):
            control('show')
            
        def on_show(self, event):
            control('show')
            
        def on_hide(self, event):
            control('hide')
            
        def on_restart(self, event):
            control('restart')
            
        def on_exit(self, event):
            control('exit')
            
        def on_reconnect(self, event):
            control('reconnect')
            
        def on_toolbar(self, event):
            control('toolbar')
        
        def select_icon(self, icon_name):
            if icon_name in self.icons.keys():
                self.current = icon_name
                self.SetIcon(self.icons.get(self.current, self.icons.values()[0]), LABEL)
                    
    class MyApp(wx.App):
        def __init__(self, icons_path, icons_dict):
            self.icons_path = icons_path
            self.icons_dict = icons_dict
            wx.App.__init__(self, False)
            
        def OnInit(self):
            self.trayicon = MyTaskBarIcon(self.icons_path, self.icons_dict)
            return True
        
        def OnExit(self):
            self.trayicon.Destroy() 
            
        def SetIcon(self, name):
            self.trayicon.select_icon(name)
    
        
    _IconObject = MyApp(icons_path, icons_dict) 
    reactor.registerWxApp(_IconObject)


def control(cmd):
    global _ControlFunc
    if _ControlFunc is not None:
        _ControlFunc(cmd)


def set(name):
    global _IconObject
    if _IconObject is not None:
        _IconObject.SetIcon(name)
      
      
def state_changed(network, p2p, central):
    if 'DISCONNECTED' in [ network, p2p ]:
        set('gray')
        return
    if [ network, p2p, central ].count('CONNECTED') == 3:
        set('green')
        return
    set('gray')


def SetControlFunc(f):
    global _ControlFunc
    _ControlFunc = f

#------------------------------------------------------------------------------ 

if __name__ == "__main__":
    def test_control(cmd):
        print cmd
        if cmd == 'exit':
            reactor.stop()
        
    from twisted.internet import wxreactor
    wxreactor.install()
    from twisted.internet import reactor
    init(sys.argv[1])
    SetControlFunc(test_control)
    reactor.run()
    
    
    





def usage():
    return '''usage: datahaven [options] [command] [arguments]
    
Commands:
  start
  restart
  stop
  show
  register <account name>
  recover <private key filename> [idurl or username]
  key copy
  key copy <filename for copy of private key>
  key print
  backup list
  backup idlist
  backup start <folder>
  backup add <folder>
  backup delete <folder>
  backup delete <backup ID>
  backup delete local <backup ID>
  backup update
  schedule <folder> [schedule in compact format]
  restore <backup ID> [destination ".tar" file path]
  stats <folder>
  stats <backup ID>
  stats remote <backup ID>
  stats local <backup ID>
  suppliers 
  suppliers call
  supplier replace <username, number or idurl>
  supplier change <username, number or idurl> <username or idurl>
  customers  
  customers call     
  customer remove <username or idurl>
  storage                                                         
  reconnect
  states
  money
  money transfer <username or idurl> <amount>
  money receipts 
  money receipt <receipt ID>
  set <option> [value]
  version
  help
'''    


def help():
    return '''usage: datahaven [options] [command] [arguments]

Commands:
  start                 start DataHaven.NET
  
  restart               restart DataHaven.NET 

  stop                  stop DataHaven.NET

  show                  start DataHaven.NET and show the main window

  register <account name>
                        generate a new private key and register new account

  recover <private key filename> [idurl or username]
                        recover existing account with your private key file
                        
  key copy              copy private key to clipboard to paste with Ctrl+V somewhere 
  
  key copy <a filename for copy of private key>
                        copy private key into file to save it in a safe place
                        
  key print             print private key  

  backup list           show list of your backup folders

  backup start <folder> start a new backup of the given folder

  backup add <folder>   add backup folder, but not start new backup

  backup delete <folder>
                        remove all backups of the folder

  backup delete <backup ID>
                        remove given backup

  backup delete local <backup ID>
                        remove only local files for this backup,
                        but keep "remote copy" on suppliers HDD.
                        
  backup update         request all suppliers to update info for all backups 

  schedule <folder> [schedule in compact format]
                        set or get a schedule for a folder to start backups automatically

  restore <backup ID> [.tar file location]
                        restore a given backup into .tar archive
                        default location is your home folder
                        
  stats <backup ID>     show condition of given backup
  
  stats remote <backup ID>
                        show remote files stats for this backup

  stats local <backup ID>
                        show local files stats for this backup

  suppliers             show list of your suppliers

  suppliers call        send a request packets to check out suppliers status

  supplier replace <username, number or idurl>
                        replace a single supplier with new one, given by central server

  supplier change <username, number or idurl> <username or idurl>
                        ask to change one supplier to another, by your choice
                        
  customers             show list of your customers
  
  customers call        send a request packets to check out customers status
  
  customer remove <username or idurl>
                        remove a single customer
                        
  storage               print detailed info about needed and donated space

  reconnect             restart network connection

  states                print state machines info
  
  money                 show the financial status 

  money transfer <username or idurl> <amount>
                        transfer money to another user 

  money receipts        show receipts history

  money receipt <receipt ID>
                        show single receipt info
  
  set <option> [value]  to modify program setting

  version               display current software version

  help                  print this message
  
  help schedule         print format description to set scheduled backup
  
  help settings         print settings list

'''


def schedule_format():
    return '''
Schedule compact format:
[mode].[interval].[time].[details]

mode:
  n-none, h-hourly, d-daily, w-weekly, m-monthly, c-continuously
  
interval:
  just a number - how often to restart the task, default is 1
    
time:
  [hour]:[minute]
  
details:
  for weeks: Mon Tue Wed Thu Fri Sat Sun
  for months: Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
  
some examples:
  none                    no schedule
  hourly.3                each 3 hours
  daily.4.10:15.          every 4th day at 10:15
  w.1.3:00.MonSat         every Monday and Saturday at 3:00 in the night
  weekly.4.18:45.MonTueWedThuFriSatSun
                          every day in each 4th week in 18:45
  m.5.12:34.JanJul        5th Jan and 5th July at 12:34
  c.300                   every 300 seconds (10 minutes)
'''
    
def settings_help():
    return '''set [option] [value]          

examples:
  set donated 4GB                          set donated space
  set needed                               print your needed space size
  set general.general-backups 4            set number of backup copies for every folder
  set suppliers                            print number of your suppliers
  set logs.stream-enable False             turn off web server for program logs
  set list                                 list all available options

'''


from nitro.async import AsyncOp
from nitro.tcp import tcpConnect, TCPCloser, TCPStream

class Dummy : pass

def _writeAndClose( sock, data, reactor, callback=None ) :
    def doCancel() :
        closerOp.cancel()
        sock.close()
    def onCloserDone() :
        op.notify()
    closerOp = TCPCloser( sock, data, reactor, onCloserDone ).getOp()
    op = AsyncOp( callback, doCancel )
    return op

class FirewallTestServer( object ) :
    def __init__( self, tcpListener, token, reactor ) :
        self.tcpListener = tcpListener
        self.token = token
        self.reactor = reactor
        tcpListener.enable( True )
        tcpListener.setCallback( self._onIncoming )
        self.connOps = set()

    def close( self ) :
        for connOp in self.connOps :
            connOp.cancel()
        self.connOps.clear()
        self.tcpListener.enable( False )
        self.tcpListener.setCallback( None )

    def _onIncoming( self, sock ) :
        def onComplete() :
            self.connOps.remove( connOp )
        connOp = _writeAndClose( sock, self.token, self.reactor,
                onComplete )
        self.connOps.add( connOp )

def firewallTestClient( addr, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.connectOp :
            obj.connectOp.cancel()
            obj.connectOp = None
        if obj.stream :
            obj.stream.close()
            obj.stream = None
        if obj.timerOp :
            obj.timerOp.cancel()
            obj.timerOp = None
    def doNotify() :
        doCancel()
        if obj.data :
            op.notify( 0, obj.data )
        else :
            op.notify( -1, '' )
    def onClose( *args ) :
        doNotify()
    def onInput( data ) :
        obj.data += data
        if len(obj.data) > 100 :
            obj.data = ''
            doNotify()
            return
    def onTimer() :
        obj.timerOp = None
        doNotify()
    def onConnect( connector ) :
        obj.connectOp = None
        if connector.getError() != 0 :
            doNotify()
            return
        sock = connector.getSock()
        obj.stream = TCPStream( sock, reactor )
        obj.stream.setCloseCallback( onClose )
        obj.stream.setErrorCallback( onClose )
        obj.stream.setInputCallback( onInput )
        obj.stream.initiateRead( 100 )
    obj.connectOp = tcpConnect( addr, reactor, onConnect )
    obj.timerOp = reactor.callLater( 15, onTimer )
    obj.stream = None
    obj.data = ''
    op = AsyncOp( callback, doCancel )
    return op



from socket import error as sock_error

from ncrypt0.ssl import SSLError, SSLWantReadError, SSLWantWriteError
from nitro.errors import EWOULDBLOCK, EAGAIN
from nitro.ssl import sslClose

class HandleState( object ) :
    def __init__( self, isSSL, sock=None, sslConn=None ) :
        self.isSSL = isSSL
        if isSSL :
            self.sslConn = sslConn
            self.handle = sslConn.sock
        else :
            self.sock = sock
            self.handle = sock
        self.closed = False
        self.hasReadEvent = False
        self.hasWriteEvent = False

class StreamState( object ) :
    def __init__( self, srcHandle, destHandle ) :
        self.handles = [srcHandle, destHandle]
        self.wantRead = [False,False]
        self.wantWrite = [False,False]
        self.data = ''
        self.maxData = 16384

    def empty( self ) : return len(self.data) == 0
    def size( self ) : return len(self.data)
    def remaining( self ) : return self.maxData - len(self.data)
    def addData( self, data ) : self.data += data

class SSLBridge( object ) :
    def __init__( self, sock, sslConn, reactor ) :
        self.sock = sock
        self.sslConn = sslConn
        self.reactor = reactor
        self.handles = [ HandleState(False,sock=sock), HandleState(True,sslConn=sslConn) ]
        self.streams = [
            StreamState(self.handles[0],self.handles[1]),
            StreamState(self.handles[1],self.handles[0]) ]

        for s in self.streams : s.wantRead[0] = True
        self.updateEvents()
        self.closeCallback = None

    def setCloseCallback( self, closeCallback ) :
        self.closeCallback = closeCallback

    def shutdown( self ) :
        for hs in self.handles :
            if not hs.closed :
                self.closeHandle( hs )

    def updateEvents( self ) :
        needRead = [False,False]
        needWrite = [False,False]
        h, s = self.handles, self.streams
        for i in (0,1) :
            needRead[i] = (not h[i].closed) and (s[i].wantRead[0] or s[1-i].wantRead[1])
            needWrite[i] = (not h[i].closed) and (s[i].wantWrite[0] or s[1-i].wantWrite[1])
            if needRead[i] != h[i].hasReadEvent :
                if needRead[i] :
                    def regRead( handle ) :
                        self.reactor.addReadCallback( handle.fileno(), lambda : self._onRead(handle) )
                    regRead( h[i].handle )
                else :
                    self.reactor.removeReadCallback( h[i].handle.fileno() )
                h[i].hasReadEvent = needRead[i]
            if needWrite[i] != h[i].hasWriteEvent :
                if needWrite[i] :
                    def regWrite( handle ) :
                        self.reactor.addWriteCallback( handle.fileno(), lambda : self._onWrite(handle) )
                    regWrite( h[i].handle )
                else :
                    self.reactor.removeWriteCallback( h[i].handle.fileno() )
                h[i].hasWriteEvent = needWrite[i]

    def closeHandle( self, hs ) :
        assert not hs.closed
        hs.closed = True
        if hs.hasReadEvent :
            self.reactor.removeReadCallback( hs.handle.fileno() )
            hs.hasReadEvent = False
        if hs.hasWriteEvent :
            self.reactor.removeWriteCallback( hs.handle.fileno() )
            hs.hasWriteEvent = False
        if hs.isSSL :
            sslClose( hs.sslConn, self.reactor )
        else :
            hs.sock.close()

    def pumpStream( self, stream ) :
        src = stream.handles[0]
        dest = stream.handles[1]
        finished = False
        while not finished :
            finished = True
            stream.wantRead[0] = stream.wantWrite[0] = False
            while (not src.closed) and (stream.remaining() > 0) :
                if not src.isSSL :
                    try :
                        data = src.sock.recv( stream.remaining() )
                        if not data :
                            self.closeHandle( src )
                            break
                        stream.addData( data )
                    except sock_error, (err,errMsg) :
                        if err in (EWOULDBLOCK,EAGAIN) :
                            stream.wantRead[0] = True
                        else :
                            self.closeHandle( src )
                        break
                else :
                    try :
                        data = src.sslConn.recv( stream.remaining() )
                        if not data :
                            self.closeHandle( src )
                            break
                        stream.addData( data )
                    except SSLWantReadError :
                        stream.wantRead[0] = True
                        break
                    except SSLWantWriteError :
                        stream.wantWrite[0] = True
                        break
                    except SSLError :
                        self.closeHandle( src )
                        break
            srcNoSpace = (stream.remaining() == 0)

            stream.wantRead[1] = stream.wantWrite[1] = False
            while (not dest.closed) and (stream.size() > 0) :
                if not dest.isSSL :
                    try :
                        written = dest.sock.send( stream.data )
                        stream.data = stream.data[written:]
                    except sock_error, (err,errMsg) :
                        if err in (EWOULDBLOCK,EAGAIN) :
                            stream.wantWrite[1] = True
                        else :
                            self.closeHandle( dest )
                        break
                else :
                    try :
                        written = dest.sslConn.send( stream.data )
                        stream.data = stream.data[written:]
                    except SSLWantReadError :
                        stream.wantRead[1] = True
                        break
                    except SSLWantWriteError :
                        stream.wantWrite[1] = True
                        break
                    except SSLError :
                        self.closeHandle( dest )
                        break
            if (srcNoSpace) and (stream.remaining() > 0) :
                finished = False
        if src.closed and stream.empty() and (not dest.closed) :
            assert not stream.wantRead[1]
            assert not stream.wantWrite[1]
            self.closeHandle( dest )

    def _onRead( self, handle ) :
        h = self.handles
        hs = (handle == h[0].handle) and h[0] or h[1]
        if not hs.hasReadEvent : return
        for ss in self.streams : self.pumpStream( ss )
        self.updateEvents()
        if self.closeCallback :
            if self.handles[0].closed and self.handles[1].closed :
                self.closeCallback()
                self.closeCallback = 1 # set to some non-callable

    def _onWrite( self, handle ) :
        h = self.handles
        hs = (handle == h[0].handle) and h[0] or h[1]
        if not hs.hasWriteEvent : return
        for ss in self.streams : self.pumpStream( ss )
        self.updateEvents()
        if self.closeCallback :
            if self.handles[0].closed and self.handles[1].closed :
                self.closeCallback()
                self.closeCallback = 1 # set to some non-callable



class Reactor(object) :
    def addReadCallback( self, sockfd, callback ) :
        raise NotImplementedError
    def removeReadCallback( self, sockfd ) :
        raise NotImplementedError

    def addWriteCallback( self, sockfd, callback ) :
        raise NotImplementedError
    def removeWriteCallback( self, sockfd ) :
        raise NotImplementedError

    def addExceptionCallback( self, sockfd, callback ) :
        raise NotImplementedError
    def removeExceptionCallback( self, sockfd ) :
        raise NotImplementedError

    # timeout is seconds in floating point, returns async.Op
    def addTimer( self, timeout, callback=None ) :
        raise NotImplementedError

    # single shot timer, timeout is float, return async.Op
    def callLater( self, timeout, callback=None ) :
        raise NotImplementedError



import logging
from types import IntType, ListType

from nitro.async import AsyncOp
from nitro.tcp import TCPMessageStream
from nitro.bencode import encode, decode, DecodeError

logger = logging.getLogger( 'cspace.net' )

class RPCContext(object) :
    def __init__( self, requestId, rpcConnection ) :
        self.requestId = requestId
        self.rpcConnection = rpcConnection
    def getRequestId( self ) : return self.requestId
    def getRPCConnection( self ) : return self.rpcConnection

    def response( self, data ) :
        self.rpcConnection.response( data, self.requestId )

class RPCOp(AsyncOp) :
    def __init__( self, requestId, callback=None, canceler=None ) :
        AsyncOp.__init__( self, callback, canceler )
        self.requestId = requestId
    def getRequestId( self ) : return self.requestId

class RPCConnection(object) :
    def __init__( self, sock, reactor ) :
        self.sock = sock
        self.reactor = reactor
        self.nextRequestId = 0
        self.pending = {}
        self.closeCallback = None
        self.requestCallback = None
        self.onewayCallback = None
        ms = TCPMessageStream( sock, reactor )
        self.msgStream = ms
        ms.setCloseCallback( self._onClose )
        ms.setErrorCallback( self._onError )
        ms.setInvalidMessageCallback( self._onInvalidMessage )
        ms.setInputCallback( self._onInput )
        ms.enableRead( True )
        self.hasShutdown = False

    def getSock( self ) : return self.sock

    def setRequestCallback( self, requestCallback ) :
        self.requestCallback = requestCallback
    def setOnewayCallback( self, onewayCallback ) :
        self.onewayCallback = onewayCallback
    def setCloseCallback( self, closeCallback ) :
        self.closeCallback = closeCallback
    def setWriteCompleteCallback( self, writeCompleteCallback ) :
        self.msgStream.setWriteCompleteCallback( writeCompleteCallback )

    def getPendingWrite( self ) : return self.msgStream.getPendingWrite()

    def close( self, deferred=False ) :
        self._shutdownInternal()
        if not self.msgStream.hasShutdown() :
            self.msgStream.close( deferred=deferred )

    def shutdown( self ) :
        self._shutdownInternal()
        if not self.msgStream.hasShutdown() :
            self.msgStream.shutdown()

    def _shutdownInternal( self ) :
        if self.hasShutdown : return
        self.hasShutdown = True
        while self.pending :
            (requestId,op) = self.pending.popitem()
            op.notify( -1, None )

    def _shutdownAndNotify( self ) :
        assert not self.hasShutdown
        self._shutdownInternal()
        if not self.msgStream.hasShutdown() :
            if self.closeCallback :
                self.closeCallback()
            else :
                self.close()

    def allowIncoming( self, allow ) :
        self.msgStream.enableRead( allow )

    def request( self, payload, callback=None ) :
        requestId = self.nextRequestId
        self.nextRequestId += 1
        msg = (0,requestId,payload)
        s = encode( msg )
        def cancelRequest() :
            del self.pending[requestId]
        op = RPCOp( requestId, callback, cancelRequest )
        self.pending[requestId] = op
        self.msgStream.sendMessage( s )
        return op

    def response( self, payload, requestId ) :
        msg = (1,requestId,payload)
        s = encode( msg )
        self.msgStream.sendMessage( s )

    def oneway( self, payload ) :
        requestId = self.nextRequestId
        self.nextRequestId += 1
        msg = (2,requestId,payload)
        s = encode( msg )
        self.msgStream.sendMessage( s )

    def _onClose( self ) :
        self._shutdownAndNotify()

    def _onError( self, err, errMsg ) :
        logger.warning( 'error in rpc connection(%d): %s', err, errMsg )
        self._shutdownAndNotify()

    def _onInvalidMessage( self ) :
        logger.warning( 'invalid rpc message received. closing...' )
        self._shutdownAndNotify()

    def _onInput( self, data ) :
        try :
            (msgCode,requestId,payload) = decode(data)
            if (type(msgCode) is not IntType) or (type(requestId) is not IntType) :
                raise TypeError, 'invalid msg structure'
        except DecodeError, de :
            logger.warning( 'invalid msg structure: %s', de )
            return
        except (TypeError,ValueError), e :
            logger.warning( 'invalid msg: %s', e )
            return
        if msgCode == 0 : # incoming request
            ctx = RPCContext( requestId, self )
            if self.requestCallback :
                self.requestCallback( payload, ctx )
        elif msgCode == 1 : # incoming response
            op = self.pending.get( requestId, None )
            if op is None :
                logger.warning( 'unmatched response requestId(%d)', requestId )
                return
            del self.pending[requestId]
            op.notify( 0, payload )
        elif msgCode == 2 : # incoming one-way message
            if self.onewayCallback :
                self.onewayCallback( payload )
        else :
            logger.warning( 'invalid msg code: %d', msgCode )

class RPCSwitchboard( object ) :
    def __init__( self, rpcConnection ) :
        self.rpcConnection = rpcConnection
        self.requestAgents = {}
        self.onewayAgents = {}
        self.rpcConnection.setRequestCallback( self._onRequest )
        self.rpcConnection.setOnewayCallback( self._onOneway )

    def addRequestAgent( self, agentName, agentCallback ) :
        assert agentName not in self.requestAgents
        self.requestAgents[agentName] = agentCallback

    def addOnewayAgent( self, agentName, agentCallback ) :
        assert agentName not in self.onewayAgents
        self.onewayAgents[agentName] = agentCallback

    def delRequestAgent( self, agentName ) :
        del self.requestAgents[agentName]

    def delOnewayAgent( self, agentName ) :
        del self.onewayAgents[agentName]

    def _onRequest( self, payload, ctx ) :
        if type(payload) is not ListType :
            logger.warning( 'invalid payload type: %s', str(type(payload)) )
            return
        try :
            agentName,payload = payload
        except ValueError, e :
            logger.warning( 'error decoding payload: %s', str(e) )
            return
        agentCallback = self.requestAgents.get( agentName, None )
        if agentCallback is None :
            logger.warning( 'request agent not found: %s' % agentName )
            return
        agentCallback( payload, ctx )

    def _onOneway( self, payload ) :
        if type(payload) is not ListType :
            logger.warning( 'invalid payload type: %s', str(type(payload)) )
            return
        try :
            agentName,payload = payload
        except ValueError, e :
            logger.warning( 'error decoding payload: %s', str(e) )
            return
        agentCallback = self.onewayAgents.get( agentName, None )
        if agentCallback is None :
            logger.warning( 'oneway agent not found: %s' % agentName )
            return
        agentCallback( payload )

class RPCStub( object ) :
    def __init__( self, rpcConnection, objectName ) :
        self.rpcConnection = rpcConnection
        self.objectName = objectName

    def request( self, payload, callback=None ) :
        return self.rpcConnection.request( (self.objectName,payload), callback )

    def oneway( self, payload ) :
        self.rpcConnection.oneway( (self.objectName,payload) )



#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#  http://docs.python.org/lib/tar-examples.html
#
#  This python code can be used to replace the Unix tar command
#  and so be portable to non-unix machines.
#
#  There are other python tar libraries, but this is included with python.
#
#  If we kept track of how far we were through a list of files, and broke off
#    new dhnblocks at file boundaries, we could restart a backup and continue
#    were we left off if a crash happened while we were waiting to send a block
#    (most of the time is waiting so good chance).
#
#  *************  WARNING ***************
#  Note that we should not print things here because tar output goes to standard out.
#  If we print anything else to stdout the .tar file will be ruined.
#  We must also not print things in anything this calls.
#  *************  WARNING ***************

import os
import sys
import platform
import tarfile


try:
    import msvcrt
    msvcrt.setmode(1, os.O_BINARY)
except:
    pass


def printlog(txt):
    LogFile = open('dhnbackup.log', 'a')
    LogFile.write(txt)
    LogFile.close()



def _LinuxExcludeFunction(filename): 
    # filename comes in with the path relative to the start path, 
    # so dirbeingbackedup/photos/christmas2008.jpg
    if filename.count("datahavennet"):
        return True
    if filename.count(".datahaven"):
        return True
    if not os.access(filename, os.R_OK):
        return True
    # PREPRO  
    #   On linux we should test for the attribute meaning "nodump" or "nobackup"
    # This is set with
    #     chattr +d  file
    #   And listed with 
    #     lsattr file
    #  Also should test that the file is readable and maybe that directory is executable. 
    #  If tar gets stuff it can not read it just stops.
    return False # don't exclude the file

def _WindowsExcludeFunction(filename): 
    # filename comes in with the path relative to the start path, 
    # so "Local Settings\Application Data\Microsoft\Windows\UsrClass.dat"
    # on windows I run into some files that Windows tells me 
    # I don't have permission to open (system files), 
    # I had hoped to use "os.access(filename, os.R_OK) == False" 
    # to skip a file if I couldn't read it, 
    # but I did not get it to work every time. DWC
    if (filename.lower().find("local settings\\temp") != -1) or (filename.lower().find("datahaven.net") != -1) :
        return True
    return False # don't exclude the file


_ExcludeFunction = _LinuxExcludeFunction
if platform.uname()[0] == 'Windows':
    _ExcludeFunction = _WindowsExcludeFunction

    
def writetar(filelist, subdirs=True):
    tar = tarfile.open('', "w|", fileobj=sys.stdout )

    # if we have python 2.6 then we can use an exclude function
    if sys.hexversion >= 0x2060100:
        for name in filelist:
            tar.add(name, None, subdirs, _ExcludeFunction) # the True is for recursive, if we wanted to just do the immediate directory set to False
            if not subdirs:
                for file in os.listdir(name):
                    if not os.path.isdir(os.path.join(name,file)): 
                        tar.add(os.path.join(name,file), None, subdirs, _ExcludeFunction) # the True is for recursive, if we wanted to just do the immediate directory set to False

    else: # otherwise no exclude function
        for name in filelist:
            tar.add(name, None, subdirs)
            if not subdirs:
                for file in os.listdir(name):
                    if not os.path.isdir(os.path.join(name,file)): 
                        tar.add(os.path.join(name,file), None, subdirs) 

    tar.close()

#def readtar():
#    tar = tarfile.open(mode="r|", fileobj=sys.stdin)
#    for tarinfo in tar:
#        tar.extract(tarinfo)
#    tar.close()

def main():
    if len(sys.argv) < 3:
        printlog('sys.argv: %s\n' % str(sys.argv))
        printlog('dhnbackup ["subdirs"/"nosubdirs"] [folder path]\n')
        sys.exit(2)

#    subdirs = True
#    if sys.argv[1].strip().lower() == 'nosubdirs':
#        subdirs = False

#    archiveDir = sys.argv[2]
#    for i in range(2, len(sys.argv)):
#        archiveDir = archiveDir + sys.argv[i] + ' '

    writetar([sys.argv[2]], sys.argv[1].strip().lower() != 'nosubdirs')


if __name__ == "__main__":
    main()





#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#  ECCMAP.PY
#
# This object holds one error correcting code map.
# When creating an object we give a filename to load from.
#
# The spec says for each datablock which parity blocks XOR it.
# As we read in each datablock we XOR all the parity blocks for it.
# Probably best to read in all the datablocks at once, and do all
# the parities one time.  If we did one datablock after another
# the parity blocks would need to be active all the time.
#
# If we XOR integers, we have a byte order issue probably, though
# maybe not since they all stay the same order, whatever that is.
#
# Some machines are 64-bit.  Would be fun to make use of that if
# we have it.
#
# 2^1 => 3       ^ is XOR operator
#
# Parity packets have to be a little longer so they can hold
# parities on signatures of datapackets.  So we can reconstruct
# those signatures.
#
# Numbering is from 0 to 63 for range of 64

import os
import re
#import struct

import dhnio
import settings


__suppliers2eccmap = {
2 : 'ecc/2x2',
4 : 'ecc/4x4',
7 : 'ecc/7x7',
13 : 'ecc/13x13',
18 : 'ecc/18x18',
26 : 'ecc/26x26',
64 : 'ecc/64x64',
}

__suppliers_numbers = __suppliers2eccmap.keys()
__suppliers_numbers.sort()

__eccmap2suppliers = {
'ecc/2x2': 2,
'ecc/4x4': 4,
'ecc/7x7': 7,
'ecc/13x13': 13,
'ecc/18x18': 18,
'ecc/26x26': 26,
'ecc/64x64': 64,
}

__eccmap_names = __eccmap2suppliers.keys()
__eccmap_names.sort()



__eccmaps = {
'ecc/2x2': [[1],[0]],
'ecc/4x4': [[1,2,3],[0,2],[0,3],[0,1]],
'ecc/7x7': [[3,4,6],[0,4,5],[1,5,6],[0,2,6],[0,1,3],[1,2,4],[2,3,5]],
'ecc/13x13': [[1,4,8,12],[5,8,9,11],[3,7,10,11],[0,4,6,9],[2,3,6,12],
    [0,1,6,10],[1,3,7,9],[2,5,8,12],[2,4,7,11],[0,1,3,5,12],[6,7,8],[2,5,9,10],[0,4,10,11]],
'ecc/18x18': [[5,7,11,16,17],[2,9,11,13,17],[5,8,9,13,15],[0,1,4,6,10],[2,3,12,13,14],
    [6,8,13,17],[2,5,10,12],[3,10,11,14],[0,1,3,4,5,6,7,9,10,11,13,14,15,16,17],
    [0,1,12,14],[5,6,8,14,16],[0,4,7,9],[2,4,7,8],[3,4,6,11,15],
    [0,10,15,16],[1,2,17],[3,8,12,15],[1,7,9,12,16]],
'ecc/26x26': [[1,8,11,16,19,21],[3,6,8,17,23],[6,7,11,17,21,25],
    [0,10,13,14,21],[5,9,10,18,22],[12,13,17,20,21,22],[1,2,9,13],
    [2,3,5,9,20,22],[0,6,9,12,15,25],[2,7,14,15,16,24],[2,5,6,11,15,16,18,19,23],
    [2,10,12,13,14,20,23],[0,3,4,11,19],[0,1,4,18,19,20,23,25],[1,5,7,11,20,21,25],
    [1,4,16,17,18],[2,4,11,22,24],[5,12,13,14,16,24],[3,7,10,20,22,24,25],
    [0,8,10,12,17],[0,8,9,17,19,22,25],[4,5,15,16,22],[6,8,12,14,15,18,23],
    [1,3,7,13,19,24],[0,3,4,7,14,15,21,23],[6,8,9,10,18,24]],
'ecc/64x64': [[5,17,18,31,39,47,55,58],[0,3,4,25,27,32,34,48,53,56,63],
    [10,11,17,18,25,32,36,40,45,51],[1,21,23,27,30,35,43,47,62],
    [2,19,20,21,28,29,37,38,40,55,56,62],[15,17,19,20,31,45,46,54,57,63],
    [19,20,30,36,46,47,52,62],[2,5,16,18,19,37,48,55],
    [1,2,7,12,13,20,26,28,48,55],[0,1,15,21,24,33,36,41,56,62],
    [19,20,28,30,43,45,52,57,59],[2,6,12,20,34,58,61,63],
    [5,6,13,15,25,34,36,40,42,43,50,51,55,61,62],[21,22,23,34,39,41,43,45,49,52,53,58],
    [0,12,17,19,28,57,58,63],[8,18,25,29,34,49,52,53,56,62],
    [3,6,19,23,35,39,40,43,49,54,57],[2,3,8,9,30,31,47,54,58,62],
    [0,8,14,24,28,33,36,47,52,58],[8,10,13,22,25,27,32,35,40,51,56],
    [2,14,16,17,26,27,29,31,43,46,54,56],[22,25,37,41,45,52,61],[5,9,13,32,46,50,54,62],
    [0,4,5,10,15,16,26,36,37,48,50],[13,14,20,21,40,42,55,60],[1,2,13,15,16,19,26,30,37,42,48,50,59],
    [4,10,11,18,28,30,44,45,46,60,63],[2,6,16,22,24,38,41,53,59],
    [6,15,21,23,26,29,32,34,35,36,38,43,51,54,60],[13,24,32,33,34,41,46,52,58,61],
    [1,10,23,24,27,29,40,41,61],[4,5,6,10,14,42,44,48,51,53,61],
    [0,5,7,15,49,50],[8,29,35,36,43,47,51,60,62],[7,12,15,21,22,27,31,33,57,60],
    [5,16,18,24,26,33,38,44,46,53,56,57,61],[1,3,4,9,24,27,31,39,50,51,54,58],
    [12,18,22,23,27,35,36,44,60,63],[0,12,17,20,32,35,37,50,53,59],
    [8,11,14,16,22,24,35,36,41,42,44,46,57],[14,23,30,33,34,38,42,44,46,48,54],
    [9,14,27,31,33,35,49,51,52,54],[3,8,11,12,14,30,32,34,48,56,62],[7,9,29,44,46,58],
    [6,18,21,26,28,39,40,45,47,55,58,63],[4,17,21,26,30,34,54,61],[0,5,6,10,23,29,39,55,60],
    [7,9,10,11,12,18,25,26,29,37,38,39,42,45,49],[6,7,17,27,33,56,59,60],
    [1,3,9,14,20,28,42,47,57,63],[11,17,23,25,39,41,45,53,56,57,60,61,63],
    [4,8,12,16,19,28,31,32,47],[2,4,22,23,26,39,41,42,51,59],
    [0,3,9,13,25,40,43],[0,9,10,16,22,47,53,55],
    [1,3,4,7,13,20,21,25,49,50],[6,12,15,16,17,29,33,38,48,50,55,57,59],
    [1,15,24,28,37,40,42,52],[1,4,7,13,14,30,38,59],
    [11,31,33,37,44,49,51,52],[8,11,24,31,32,35,50,53,59,63],
    [3,8,11,18,22,38,44,49],[7,9,10,19,37,41,44,45,49,60,61],[2,3,5,7,11,38,39,43,48,59]],
}

__correctable_errors = {
64: 10,
26: 6,
18: 5,
13: 4,
7: 3,
4: 2,
2: 1,
}



CurrentMap = None

def DefaultName():
    return settings.DefaultEccMapName()

def CurrentName():
    return settings.getECC()

def Current():
    global CurrentMap
    if CurrentMap is None:
        init()
    return CurrentMap

def SuppliersNumbers():
    global __suppliers_numbers
    return __suppliers_numbers
##    global __suppliers2eccmap
##    return __suppliers2eccmap.keys()

def EccMapNames():
    global __eccmap_names
    return __eccmap_names

def GetEccMapName(suppliers_number):
    global __suppliers2eccmap
##    print 'GetEccMapName'
##    print suppliers_number, type(suppliers_number), __suppliers2eccmap
    return __suppliers2eccmap.get(suppliers_number, settings.DefaultEccMapName())

def GetEccMapSuppliersNumber(eccmapname):
    global __eccmap2suppliers
    try:
        return int(__eccmap2suppliers.get(eccmapname, settings.DefaultSuppliersNumber()))
    except:
        return settings.DefaultSuppliersNumber()

def GetEccMapData(name):
    global __eccmaps
    return __eccmaps.get(name, None)

def GetCorrectableErrors(suppliers_number):
    global __correctable_errors
    return __correctable_errors.get(suppliers_number, 1)

def init():
    global CurrentMap
    if CurrentMap is not None:
        del CurrentMap
        CurrentMap = None
    CurrentMap = eccmap(CurrentName())


class eccmap:
    def __init__ (self, filename='', suppliers_number=2):
        if filename != '':
            self.name = filename      # sometimes we will just want the name
            self.suppliers_number = GetEccMapSuppliersNumber(self.name)
        else:
            self.suppliers_number = suppliers_number
            self.name = GetEccMapName(self.suppliers_number)
        self.CorrectableErrors = self.CalcCorrectableErrors(self.name)
        self.ParityToData = []    # index with Parity number and output is Data numbers for that parity
        self.DataToParity = []    # index with Data number and output is Parities that use that Data number
        self.datasegments = 0     #
        self.paritysegments = 0   #
        self.type = 0             # 0 is data+parity on same nodes, 1 is different
        self.from_memory(self.name)
        self.convert()
        
    def __repr__(self):
        return '%s' % (self.name)

    def NumSuppliers(self):
        dhnio.Dprint(4, 'eccmap.NumSuppliers ' + str(self.suppliers_number))
        return self.suppliers_number

    def DataNeeded(self):           
        # How many good segments ensures we can fix all
        return self.datasegments - self.CorrectableErrors

    def CalcCorrectableErrors(self, filename):  
        # We can fix at least this many errors (probably more for big nums)
        basename = os.path.basename(filename)
        # all our codes can handle at least one error
        CE=1                                    
        if basename=="64x64":
            CE=10
        if basename=="26x26":
            CE=6
        if basename=="18x18":
            CE=5
        if basename=="13x13":
            CE=4
        if basename=="7x7":
            CE=3
        if basename=="4x4":
            CE=2
        if basename=="2x2":
            CE=1
        return CE

    def from_memory(self, name):
        dhnio.Dprint(6, "eccmap.from_memory with " + name)
        maxdata = 0
        maxparity = 0
        data = GetEccMapData(name)
        self.ParityToData = []
        for PS in data:
            oneset=[]
            for datanum in PS:
                try:
                    oneset.append(datanum)
                    if datanum > maxdata:
                        maxdata = datanum
                except (TypeError, ValueError):
                    dhnio.Dprint(1, 'eccmap.from_memory ERROR')
                    dhnio.DprintException()
            if oneset:
                self.ParityToData.append(oneset)
                maxparity += 1
        self.paritysegments = maxparity
        self.datasegments = maxdata + 1
        # we only do this type at the moment
        self.type = 0                    
        dhnio.Dprint(6, "   %s with parity=%s data=%s " % (name, self.paritysegments, self.datasegments))


    def loadfromfile(self, fname):
        dhnio.Dprint(10, "eccmap.loadfromfile with " + fname)
        if os.path.exists(fname):
            filename = fname
        else:
            filename = os.path.join("..", fname)
        maxdata = 0
        maxparity = 0
        s = dhnio.ReadTextFile(filename)
        for PS in re.split("\]",s):
            good = PS.lstrip("\[\,")
            oneset = []
            for datastring in good.split(","):
                try:
                    datanum = int(datastring)
                    oneset.append(datanum)
                    if datanum > maxdata:
                        maxdata = datanum
                except (TypeError, ValueError):
                    dhnio.DprintException()
            if oneset:
                self.ParityToData.append(oneset)
                maxparity += 1
        self.paritysegments = maxparity
        self.datasegments = maxdata + 1
        self.type = 0                    # we only do this type at the moment
        dhnio.Dprint(10, "eccmap.loadfromfile  %s  with parity=%s  data=%s " % (filename, self.paritysegments, self.datasegments))

    def convert(self):
        for datanum in range(self.datasegments):
            self.DataToParity.append([])
        paritynum = 0
        for parity in self.ParityToData:
            for datanum in parity:
                self.DataToParity[datanum].append(paritynum)
            paritynum += 1

    def nodes(self):                  # number of nodes/hosts/sites used for this map
        if self.type == 0:            # 0 is data+parity on same nodes, 1 is different
            return self.datasegments
        else:
            return self.datasegments + self.paritysegments

    def check(self):
        for emap in self.DataToParity:
            for parity in emap:
                if parity > self.paritysegments:
                    return False
        for emap in self.ParityToData:
            for data in emap:
                if data > self.datasegments:
                    return False
        return True

    def FixableNode(self, NodeMap):              
        # case where each node has one parity and one data so same missing
        DataSegs=[0]*self.datasegments
        ParitySegs=[0]*self.datasegments
        if len(NodeMap) != self.datasegments:
            raise Exception("eccmap.FixableNode not given NodeMap of correct length")
        if self.datasegments != self.paritysegments:
            raise Exception("eccmap.FixableNode only usable if same number of data and parity ")
        for i in range(0, self.datasegments):
            DataSegs[i]=NodeMap[i]
            ParitySegs[i]=NodeMap[i]
        return(self.Fixable(DataSegs, ParitySegs))

    def Fixable(self, DataSegs, ParitySegs):         
        # Lists are 1 for Data and Parity, lists are [0,1,1,1,0...] 0 is don't have 1 is have
        stillMissing = 0
        for i in range(self.datasegments):
            if DataSegs[i] != 1:
                stillMissing += 1

        MakingProgress = 1
                                                    # now we test to see if this is fixable
        while MakingProgress == 1 and stillMissing > 0: 
            MakingProgress = 0
            for paritynum in range(self.paritysegments):  
                if ParitySegs[paritynum] == 1:      # foreach parity
                    Parity = self.ParityToData[paritynum] # If parity is not missing (so we can use it)
                    missing = 0                     # We will see how many of the datas are missing
                    for DataNum in Parity:          # look at all datas that went into this parity
                        if DataSegs[DataNum] == 0:  # if this data is missing
                            missing += 1            #     increase count of those missing in this parity
                            lastMissing = DataNum   #     keep track of the last missing in case only one is missing
                    if missing == 1:                # if missing exactly 1 of datas in parity
                        MakingProgress = 1          #   then we are making progress
                        DataSegs[lastMissing] = 1   #   as we could fix lastMissing with current Parity
                        stillMissing -= 1           #   so one less stillMissing

        AllFixed = stillMissing == 0                # If nothing else missing we are good
        return AllFixed

    def CanMakeProgress(self, DataSegs, ParitySegs): # Lists are 1 for Data and Parity, lists are [0,1,1,1,0...] 0 is don't have 1 is have
        for paritynum in range(self.paritysegments): #  foreach parity
            if ParitySegs[paritynum] == 1:      # If parity is not missing (so we can use it)
                Parity = self.ParityToData[paritynum]
                missing = 0                     # We will see how many of the datas are missing
                for DataNum in Parity:          # look at all datas that went into this parity
                    if DataSegs[DataNum] != 1:  # if this data is missing
                        missing += 1            #     increase count of those missing in this parity
                                                #     keep track of the last missing in case only one is missing
                if missing == 1:                # if missing exactly 1 of datas in parity, we can fix the data, have work to do
                    return True
            else:                               # we're missing this parity, can we rebuild it
                Parity = self.ParityToData[paritynum]
                missing = 0                     # We will see how many of the datas are missing
                for DataNum in Parity:          # look at all datas that went into this parity
                    if DataSegs[DataNum] != 1:  # if this data is missing
                        missing += 1            #     increase count of those missing in this parity
                                                #     keep track of the last missing in case only one is missing
                if missing == 0:                # if missing none of the data for this parity, we have work to do
                    return True
        return False

    # given a missing segment number (DataSegNum) and a list of available Data Segments and Parity Segments
    # identify which Parity to use to rebuild the missing Data Segment, return the parity segment number and
    # the map of data segments in that parity
    def GetDataFixPath(self, DataSegs, ParitySegs, DataSegNum):
        #dhnio.Dprint(14, 'eccmap.GetDataFixPath %s %s %s' % (str(DataSegNum), str(DataSegs), str(ParitySegs)))
        bestParityNum = -1
        bestParityMap = []
        if DataSegs[DataSegNum] == 1 :           # we have the data segment nothing to fix, unclear why this was called, don't expect this to happen
            #dhnio.Dprint(12, "eccmap.GetDataFixPath we already have the data segment requested to fix?")
            return bestParityNum, bestParityMap
        for paritynum in range(0, self.paritysegments):
            if ((ParitySegs[paritynum]==1) or (ParitySegs[paritynum]==True)) and \
                    (DataSegNum in self.ParityToData[paritynum]): # If parity is not missing (so we can use it) and contains the missing DataSegNum
                Parity=self.ParityToData[paritynum]
                missing=0                       # We will see how many of the datas are missing
                for DataNum in Parity:          # look at all datas that went into this parity
                    if (DataSegs[DataNum]==0):  # if this data is missing
                        missing += 1            #     increase count of those missing in this parity
                if (missing == 1):              # if missing exactly 1 of datas in parity
                    if (len(bestParityMap) == 0) or (len(Parity) < len(bestParityMap)):
                        bestParityNum = paritynum
                        bestParityMap = Parity
        return bestParityNum, bestParityMap



def main2():
    myecc = eccmap("ecc/64x64")
    print myecc.ParityToData
    print myecc.DataToParity
    myecc.check()

    print "Do some checks for rebuilding a block"
    dataSegments = [1] * 64
    paritySegments = [1] * 64
    dataSegments[2] = 0   # making it so we're missing data segment 2
    parityNum, parityMap = myecc.GetDataFixPath(dataSegments, paritySegments, 2)
    print parityNum       # as of the writing, the best parity was 7
    print parityMap
    paritySegments[7] = 0 # remove parity 7 and then try it again.
    parityNum, parityMap = myecc.GetDataFixPath(dataSegments, paritySegments, 2)
    print parityNum       # should be 11
    print parityMap


def main():
    myecc = eccmap("ecc/4x4")
    print myecc.ParityToData
    print myecc.DataToParity
    myecc.check()

    print "Do some checks for rebuilding a block"
    dataSegments = [1, 0, 1, 1]
    paritySegments = [1, 1, 1, 0]
    parityNum, parityMap = myecc.GetDataFixPath(dataSegments, paritySegments, 1)
    print parityNum       # as of the writing, the best parity was 7
    print parityMap
    print myecc.Fixable(dataSegments, paritySegments)


if __name__ == '__main__':
    main()





#!/usr/bin/python
#diskspace.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#


_Suffixes = {
    '':                 1.0,
    'bytes':            1.0,
    'b':                1.0,
    'KB':               1024.0,
    'Kb':               1024.0,
    'kb':               1024.0,
    'Kilobytes':        1024.0,
    'kilobytes':        1024.0,
    'MB':               1024.0*1024.0,
    'Mb':               1024.0*1024.0,
    'mb':               1024.0*1024.0,
    'Megabytes':        1024.0*1024.0,
    'megabytes':        1024.0*1024.0,
    'GB':               1024.0*1024.0*1024.0,
    'Gb':               1024.0*1024.0*1024.0,
    'gb':               1024.0*1024.0*1024.0,
    'Gigabytes':        1024.0*1024.0*1024.0,
    'gigabytes':        1024.0*1024.0*1024.0,
}

_SuffixLabels = ['bytes', 'Kb', 'Mb', 'Gb']

_MultiDict = {
        'Kb' : 1024.0,
        'Mb' : 1024.0 * 1024.0,
        'Gb' : 1024.0 * 1024.0 * 1024.0,
    }


class DiskSpace:
    def __init__(self, v = None, s = '0Mb'):
        if v is None:
            self.v = s
            if self.getSZ() not in ['Kb', 'Mb', 'Gb']:
                self.addSZ('Mb')
        else:
            self.v = str(self.getValueBest(v))
        self.b = self.getValue() * _MultiDict[self.getSZ()]
    def __repr__(self):
        return self.v
    def addSZ(self, sz):
        self.v += sz
    def setSZ(self, sz):
        self.v[-2] = sz[0]
        self.v[-1] = sz[1]
    def getSZ(self):
        return self.v[-2:]
    def getValue(self):
        try:
            return float(self.v[:-2])
        except:
            return 0.0
    def getValueBytes(self):
        return self.b
    def getValueKb(self):
        sz = self.getSZ()
        if sz == 'Gb':
            return round(self.getValue() * 1024.0 * 1024.0, 2)
        elif sz == 'Mb':
            return round(self.getValue() * 1024.0, 2)
        return self.getValue()
    def getValueMb(self):
        sz = self.getSZ().lower()
        if sz == 'Kb':
            return round(self.getValue() / 1024.0, 2)
        elif sz == 'Gb':
            return round(self.getValue() * 1024.0, 2)
        return self.getValue()
    def getValueGb(self):
        sz = self.getSZ()
        if sz == 'Kb':
            return round( self.getValue() / (1024.0 * 1024.0), 2)
        elif sz == 'Mb':
            return round( self.getValue() / 1024.0, 2)
        return self.getValue()
    def getValueBest(self, value = None):
        if value is not None:
            v = value
        else:
            v = self.getValueBytes()
        if v > _MultiDict['Gb']:
            res = round(v / _MultiDict['Gb'], 2)
            sz = 'Gb'
        elif v > _MultiDict['Mb']:
            res = round(v / _MultiDict['Mb'], 2)
            sz = 'Mb'
        else:
            res = round(v / _MultiDict['Kb'], 2)
            sz = 'Kb'
        return str(res)+sz


def SuffixIsCorrect(suffix):
    return suffix in _Suffixes.keys()

def SuffixLabels():
    return _SuffixLabels

def SameSuffix(suf1, suf2):
    if not SuffixIsCorrect(suf1):
        return False
    if not SuffixIsCorrect(suf2):
        return False
    return _Suffixes[suf1] == _Suffixes[suf2]

def MakeString(value, suf):
    if not SuffixIsCorrect(suf):
        return str(value)
    if round(float(value)) == float(value):
        return str(int(float(value))) + ' ' + suf
    return str(value) + ' ' + suf

# return (number, suffix) or (None, None)
# "342.67Mb" will return (342.67, "Mb")
def SplitString(s):
    num = s.rstrip('bytesBYTESgmkGMK ')
    suf = s.lstrip('0123456789., ').strip()
    try:
        num = float(num)
    except:
        return (None, None)

    if round(num) == num:
        num = int(num)

    if not SuffixIsCorrect(suf):
        return (None, None)

    return (num, suf)

# Make a correct value with siffix.
# 123456.789 will return "120.56 Kb"
# 123.456789 will return "123 bytes"
def MakeStringFromBytes(value):
    try:
        v = float(value)
    except:
        return value

    if v > _Suffixes['GB']:
        res = round(v / _Suffixes['GB'], 2)
        sz = 'GB'
    elif v > _Suffixes['MB']:
        res = round(v / _Suffixes['MB'], 2)
        sz = 'MB'
    elif v > _Suffixes['KB']:
        res = round(v / _Suffixes['KB'], 2)
        sz = 'KB'
    else:
        res = int(v)
        sz = 'bytes'
    return MakeString(res, sz)

# return total bytes from string with suffix or None
# "123.456 Mb" will return 129452998.656
def  GetBytesFromString(s, default=None):
    num, suf = SplitString(s)
    if num is None:
        return default
    return int(num * _Suffixes[suf])

# convert input string to a string with given suffix
# ("12.345 Mb", "Kb") will return "12641.28 Kb"
def MakeStringWithSuffix(s, suffix):
    b = GetBytesFromString(s)
    if b is None:
        return s
    if not SuffixIsCorrect(suffix):
        return s
    res = round(b / _Suffixes[suffix], 2)
    if _Suffixes[suffix] == 1.0:
        res = int(res)
    return MakeString(res, suffix)

def GetMegaBytesFromString(s):
    b = GetBytesFromString(s)
    if b is None:
        return None
    return round(b/(1024*1024), 2)

def MakeStringFromString(s):
    value, suf = SplitString(s)
    if value is None:
        return s
    return MakeString(value, suf) 




_hexChars = '0123456789abcdef'
_hexTab = {}
def _initHexTab() :
    for (i,c) in enumerate(_hexChars) :
        _hexTab[c] = i
        _hexTab[c.upper()] = i
_initHexTab()


class HexDecodeError( Exception ) : pass

def hexByteEncode( c ) :
    x = ord(c)
    return _hexChars[x >> 4] + _hexChars[x&15]

def hexByteDecode( s, i=0 ) :
    try :
        return chr( _hexTab[s[i]]*16 + _hexTab[s[i+1]] )
    except (KeyError,IndexError) :
        raise HexDecodeError

def hexEncode( s ) :
    return ''.join( [hexByteEncode(c) for c in s] )

def hexDecode( s ) :
    return ''.join( [hexByteDecode(s,i) for i in range(0,len(s),2)] )



import logging
from StringIO import StringIO
from socket import socket, AF_INET, SOCK_DGRAM, inet_aton
from socket import IPPROTO_IP, IP_ADD_MEMBERSHIP, IP_DROP_MEMBERSHIP
from socket import error as sock_error
from urlparse import urljoin
from elementtree import ElementTree
from nitro.async import AsyncOp
from nitro.http import HttpRequest

logger = logging.getLogger( 'nitro.upnp' )

UPNP_MCAST_GROUP = '239.255.255.250'
UPNP_PORT = 1900
UPNP_SERVICE_TYPE = 'urn:schemas-upnp-org:device:InternetGatewayDevice:1'
UPNP_SEARCH_MSG = (
        'M-SEARCH * HTTP/1.1\r\n' +
        'Host:%s:%d\r\n' % (UPNP_MCAST_GROUP,UPNP_PORT) +
        'ST:%s\r\n' % UPNP_SERVICE_TYPE +
        'Man:"ssdp:discover"\r\n' +
        'MX:3\r\n' +
        '\r\n' )
UPNP_SERVICES = ['urn:schemas-upnp-org:service:WANIPConnection:1',
                'urn:schemas-upnp-org:service:WANPPPConnection:1']

class UPnpError( Exception ) : pass

class UPnpActionArg( object ) :
    def __init__( self, name, direction ) :
        assert direction in ('in','out')
        self.name = name
        self.direction = direction

class UPnpAction( object ) :
    def __init__( self, name, args ) :
        self.name = name 
        self.args = args

class UPnpDevice( object ) :
    def __init__( self, scpdUrl, controlUrl, service ) :
        self.scpdUrl = scpdUrl
        self.controlUrl = controlUrl
        self.service = service
        self.friendlyName = None
        self.manufacturer = None
        self.actions = []

    def getAction( self, name ) :
        for action in self.actions :
            if action.name == name : return action
        return None

    def getActionPayload( self, name, params ) :
        action = self.getAction( name )
        params = dict(params)
        inArgs = []
        for arg in action.args :
            if arg.direction == 'in' :
                value = params.pop( arg.name, None )
                if value is None :
                    raise UPnpError, 'missing parameter: %s' % arg.name
                inArgs.append( (arg.name,str(value)) )
        if params :
            raise UPnpError, 'invalid parameter(s): %s' % str(params.keys())
        out = StringIO()
        sns = 'http://schemas.xmlsoap.org/soap/envelope/'
        enc = 'http://schemas.xmlsoap.org/soap/encoding/'
        print>>out, '<s:Envelope xmlns:s="%s" s:encodingStyle="%s">' % (sns,enc)
        print>>out, '<s:Body>'
        print>>out, '<u:%s xmlns:u="%s">' % (name,self.service)
        for arg,value in inArgs :
            print>>out, '<%s>%s</%s>' % (arg,value,arg)
        print>>out, '</u:%s>' % name
        print>>out, '</s:Body>'
        print>>out, '</s:Envelope>'
        return out.getvalue()

    def _onActionResponse( self, op, name, result, data ) :
        if result != 200 :
            logger.info( 'action response error: http request failed' )
            op.notify( -1, None )
            return
        def check( cond, errMsg ) :
            if not cond : raise UPnpError, errMsg
        try :
            input = StringIO( data )
            try :
                root = ElementTree.parse(input).getroot()
            except :
                logger.exception( 'error parsing xml' )
                raise UPnpError, 'invalid response xml'
            try :
                sns = root.tag[root.tag.index('{')+1:root.tag.index('}')]
            except ValueError :
                raise UPnpError, 'no soap namespace found'
            check( root.tag == '{%s}Envelope'%sns, 'unknown root element' )
            body = root.find('{%s}Body'%sns)
            check( body is not None, 'soap body not found' )
            response = body.find( '{%s}%sResponse' % (self.service,name) )
            check( response is not None, 'action response not found' )
            action = self.getAction( name )
            check( action is not None, 'invalid action name' )
            responseArgs = dict( [(arg.tag,arg.text or '') for arg in response] )
            outArgs = []
            for arg in action.args :
                if arg.direction == 'out' :
                    value = responseArgs.pop( arg.name, None )
                    if value is not None :
                        outArgs.append( (arg.name,value) )
        except UPnpError, errMsg :
            logger.warning( 'action response error: %s', errMsg )
            op.notify( -1, None )
            return
        op.notify( 0, outArgs )

    def callAction( self, name, params, reactor, callback=None ) :
        payload = self.getActionPayload( name, params )
        http = HttpRequest( reactor )
        http.addHeader( 'Content-Type: text/xml; charset="utf-8"' )
        http.addHeader( 'SOAPACTION: "%s#%s"' % (self.service,name) )
        def onResponse( result, data ) :
            self._onActionResponse( op, name, result, data )
        httpOp = http.post( self.controlUrl, payload, onResponse )
        op = AsyncOp( callback, httpOp.cancel )
        return op

class UPnpFinder( object ) :
    def __init__( self, reactor ) :
        self.reactor = reactor
        self.discoverCallback = None
        self.httpOps = {}
        self.sock = socket( AF_INET, SOCK_DGRAM )
        self.sock.bind( ('',0) )
        try :
            addr = inet_aton( UPNP_MCAST_GROUP )
            interface = inet_aton( '0.0.0.0' )
            self.sock.setsockopt( IPPROTO_IP, IP_ADD_MEMBERSHIP,
                    inet_aton(UPNP_MCAST_GROUP)+inet_aton('0.0.0.0') )
        except sock_error, (err,errMsg) :
            logger.exception( 'socket error(%d): %s', err, errMsg )
        self.reactor.addReadCallback( self.sock.fileno(), self._onRead )

    def setDiscoverCallback( self, discoverCallback ) :
        self.discoverCallback = discoverCallback

    def close( self ) :
        for op in self.httpOps.keys() :
            op.cancel()
        self.httpOps.clear()
        self.reactor.removeReadCallback( self.sock.fileno() )
        self.sock.close()
        self.sock = None

    def sendQuery( self ) :
        dest = (UPNP_MCAST_GROUP,UPNP_PORT)
        try :
            self.sock.sendto( UPNP_SEARCH_MSG, dest )
        except sock_error, (err,errMsg) :
            logger.exception( 'socket error(%d): %s', err, errMsg )

    def _onRead( self ) :
        try :
            data,fromaddr = self.sock.recvfrom( 8192 )
        except sock_error, (err,errMsg) :
            logger.exception( 'socket error(%d): %s', err, errMsg )
            return
        lines = data.splitlines()
        if not lines : return
        header = lines[0].split(None,2)
        if len(header) != 3 : return
        httpVersion,httpCode,httpMsg = header
        if not httpVersion.startswith('HTTP/') : return
        if httpCode != '200' : return
        location = None
        for line in lines[1:] :
            header = line.split(':',1)
            if len(header) != 2 : break
            name,value = header
            if name.lower() == 'location' :
                location = value.strip()
                break
        if location is None : return
        def onResult( result, data ) :
            self._onHttpGet( httpOp, result, data, location )
        httpOp = HttpRequest( self.reactor ).get( location, onResult )
        self.httpOps[httpOp] = 1

    def _onHttpGet( self, httpOp, result, data, httpUrl ) :
        del self.httpOps[httpOp]
        if result != 200 : return
        assert data is not None
        input = StringIO( data )
        try :
            root = ElementTree.parse(input).getroot()
        except :
            logger.exception( 'error parsing xml' )
            return
        try :
            ns = root.tag[root.tag.index('{')+1:root.tag.index('}')]
        except ValueError :
            logger.exception( 'no namespace found' )
            return
        baseUrl = ''
        scpdUrl = None
        controlUrl = None
        service = None
        friendlyName = None
        manufacturer = None
        for elem in root.findall('.//{%s}friendlyName'%ns) :
            friendlyName = elem.text
            break
        for elem in root.findall('.//{%s}manufacturer'%ns) :
            manufacturer = elem.text
            break
        for elem in root.findall('.//{%s}URLBase'%ns) :
            baseUrl = elem.text
            break
        for elem in root.findall('.//{%s}service'%ns) :
            serviceTypeElem = elem.find( '{%s}serviceType'%ns )
            if serviceTypeElem is None : continue
            service = serviceTypeElem.text
            if service not in UPNP_SERVICES : continue
            scpdUrlElem = elem.find( '{%s}SCPDURL'%ns )
            if scpdUrlElem is None : continue
            controlUrlElem = elem.find( '{%s}controlURL'%ns )
            if controlUrlElem is None : continue
            scpdUrl = urljoin( baseUrl, scpdUrlElem.text )
            controlUrl = urljoin( baseUrl, controlUrlElem.text )
            break
        if not scpdUrl : return
        if not controlUrl : return
        if not scpdUrl.startswith('http:') :
            scpdUrl = urljoin( httpUrl, scpdUrl )
        if not controlUrl.startswith('http:') :
            controlUrl = urljoin( httpUrl, controlUrl )
        device = UPnpDevice( scpdUrl, controlUrl, service )
        device.friendlyName = friendlyName
        device.manufacturer = manufacturer
        def onResult( result, data ) :
            self._onScpdGet( httpOp, device, result, data )
        httpOp = HttpRequest( self.reactor ).get( scpdUrl, onResult )
        self.httpOps[httpOp] = 1

    def _onScpdGet( self, httpOp, device, result, data ) :
        del self.httpOps[httpOp]
        if result != 200 : return
        assert data is not None
        input = StringIO( data )
        try :
            root = ElementTree.parse(input).getroot()
        except :
            logger.exception( 'error parsing xml' )
            return
        try :
            ns = root.tag[root.tag.index('{')+1:root.tag.index('}')]
        except ValueError :
            logger.exception( 'no namespace found' )
            return
        actionElements = root.findall('.//{%s}action'%ns)
        for actionElem in actionElements :
            nameElem = actionElem.find('{%s}name'%ns)
            if nameElem is None : continue
            actionName = nameElem.text
            actionArgs = []
            for arg in actionElem.findall('.//{%s}argument'%ns) :
                nameElem = arg.find('{%s}name'%ns)
                if nameElem is None : continue
                argName = nameElem.text
                direction = 'in'
                dirElem = arg.find('{%s}direction'%ns)
                if dirElem is not None :
                    direction = dirElem.text.lower()
                    if direction not in ('in','out') : continue
                actionArg = UPnpActionArg( argName, direction )
                actionArgs.append( actionArg )
            action = UPnpAction( actionName, actionArgs )
            device.actions.append( action )
        if self.discoverCallback :
            self.discoverCallback( device )

class UPnpActions( object ) :
    def findDevice( reactor, callback=None ) :
        def doCancel() :
            finder.close()
            timerOp.cancel()
        def onDiscover( device ) :
            finder.close()
            timerOp.cancel()
            op.notify( device )
        def onTimeout() :
            finder.close()
            op.notify( None )
        finder = UPnpFinder( reactor )
        finder.setDiscoverCallback( onDiscover )
        finder.sendQuery()
        finder.sendQuery()
        timerOp = reactor.callLater( 20, onTimeout )
        op = AsyncOp( callback, doCancel )
        return op
    findDevice = staticmethod( findDevice )

    def getExternalIP( device, reactor, callback=None ) :
        def onResponse( result, data ) :
            if result < 0 :
                op.notify( None )
                return
            externalIP = dict(data).get('NewExternalIPAddress',None)
            op.notify( externalIP )
        actionOp = device.callAction( 'GetExternalIPAddress', {},
                reactor, onResponse )
        op = AsyncOp( callback, actionOp.cancel )
        return op
    getExternalIP = staticmethod( getExternalIP )

    def listMappings( device, reactor, callback=None ) :
        class Dummy : pass
        obj = Dummy()
        def doCancel() :
            obj.actionOp.cancel()
        def parsePortMapping( data ) :
            data = dict( data )
            try :
                remote = data['NewRemoteHost']
                external = int( data['NewExternalPort'] )
                proto = data['NewProtocol']
                internal = int( data['NewInternalPort'] )
                client = data['NewInternalClient']
                enabled = int( data['NewEnabled'] )
                desc = data['NewPortMappingDescription']
                lease = int(data['NewLeaseDuration'])
            except (KeyError,ValueError,TypeError) :
                raise UPnpError
            return (remote,external,proto,internal,client,enabled,desc,lease)
        def onResponse( result, data ) :
            obj.actionOp = None
            try :
                if result < 0 : raise UPnpError
                mapping = parsePortMapping( data )
            except UPnpError :
                obj.op.notify( None )
                return
            obj.index += 1
            doAction()
            obj.op.notify( mapping )
        def doAction() :
            params = { 'NewPortMappingIndex':obj.index }
            obj.actionOp = device.callAction( 'GetGenericPortMappingEntry',
                    params, reactor, onResponse )
        obj.index = 0
        doAction()
        obj.op = AsyncOp( callback, doCancel )
        return obj.op
    listMappings = staticmethod( listMappings )

    def listAllMappings( device, reactor, callback=None ) :
        mappings = []
        def onResponse( mapping ) :
            if mapping is not None :
                mappings.append( mapping )
                return
            op.notify( mappings )
        listOp = UPnpActions.listMappings( device, reactor, onResponse )
        op = AsyncOp( callback, listOp.cancel )
        return op
    listAllMappings = staticmethod( listAllMappings )

    def addMapping( device, externalPort, protocol, internalPort,
            internalIP, description, reactor, callback=None ) :
        assert protocol in ('TCP','UDP')
        params = {
            'NewRemoteHost' : '',
            'NewExternalPort' : externalPort,
            'NewProtocol' : protocol,
            'NewInternalPort' : internalPort,
            'NewInternalClient' : internalIP,
            'NewEnabled' : 1,
            'NewPortMappingDescription' : description,
            'NewLeaseDuration' : 0 }
        def onResponse( result, data ) :
            op.notify( result == 0 )
        actionOp = device.callAction( 'AddPortMapping', params, reactor, onResponse )
        op = AsyncOp( callback, actionOp.cancel )
        return op
    addMapping = staticmethod( addMapping )

    def delMapping( device, externalPort, protocol, reactor, callback=None ) :
        assert protocol in ('TCP','UDP')
        params = {
            'NewRemoteHost' : '',
            'NewExternalPort' : externalPort,
            'NewProtocol' : protocol }
        def onResponse( result, data ) :
            op.notify( result == 0 )
        actionOp = device.callAction( 'DeletePortMapping', params, reactor, onResponse )
        op = AsyncOp( callback, actionOp.cancel )
        return op
    delMapping = staticmethod( delMapping )



"""
    Implementations of form input controls
"""

import os
import sys
import time

from forms import GetParam, UnpackParam, form

import wx
import wx.html


def TypeHandler(typeName):
    """ A metaclass generator. Returns a metaclass which
    will register it's class as the class that handles input type=typeName
    """
    def metaclass(name, bases, dict):
        klass = type(name, bases, dict)
        form.FormTagHandler.registerType(typeName.upper(), klass)
        return klass
    return metaclass


class FormControlMixin(object):
    """ Mixin provides some stock behaviors for
    form controls:
        Add self to the form fields
        Setting the name attribute to the name parameter in the tag
        Disabled attribute
        OnEnter and OnClick methods for binding by
        the actual control
    """
    def __init__(self, form, tag):
        if not form:
            return
        self.__form = form
        self.name = GetParam(tag, "NAME", None)
        self.target = GetParam(tag, "TARGET", self.name)
        self.id = GetParam(tag, "ID", None)
        form.fields.append(self)
        if tag.HasParam("DISABLED"):
            wx.CallAfter(self.Disable)
        # sys.stderr.write('FormControlMixin %s %s %s\n' % ( self.name, self.target, self.id))
        # sys.stderr.flush()

    def OnEnter(self, evt):
        self.__form.hitSubmitButton(self.id)

    def OnClick(self, evt):
        # self.__form.submit(self if evt else None)
        self.__form.submit(self)
        
    def GetTarget(self):
        return self.target
    
    def GetID(self):
        return self.id


_ParentWindowObject = None
class SubmitButton(wx.Button, FormControlMixin):
    __metaclass__ = TypeHandler("SUBMIT")

    def __init__(self, parent, form, tag, parser, *args, **kwargs):
        # print 'SubmitButton.__init__', time.time()
        label = GetParam(tag, "VALUE", default="Submit Query")
        kwargs["label"] = label
        kwargs["style"] = wx.BU_EXACTFIT
        wx.Button.__init__(self, parent, *args, **kwargs)
        FormControlMixin.__init__(self, form, tag)
        self.Show(False)
        # self.Lower()
        self.Parent.RegisterButton(self)
        self.SetSize((int(GetParam(tag, "SIZE", default=-1)), -1))
        self.Path = None
        self.defaultPath = GetParam(tag, "PATH", default=os.path.expanduser('~'))
        self.dialogLabel = GetParam(tag, "LABEL", default='')
        if self.name:
            if self.name.lower().strip() == 'opendir':
                self.Bind(wx.EVT_BUTTON, self.OnClickOpenDir)
            elif self.name.lower().strip() == 'openfile':
                self.Bind(wx.EVT_BUTTON, self.OnClickOpenFile)
            elif self.name.lower().strip() == 'savefile':
                self.Bind(wx.EVT_BUTTON, self.OnClickSaveFile)
            else:
                self.Bind(wx.EVT_BUTTON, self.OnClick)
        else:
            self.Bind(wx.EVT_BUTTON, self.OnClick)
    
    def __del__(self):
        # print 'SubmitButton.__del__', time.time()
        wx.Button.__del__(self)

    def OnClickOpenDir(self, evt):
        global _ParentWindowObject
        _ParentWindowObject = self.Parent
        _ParentWindowObject.BlockRepaint()
        try:
            dialog = wx.DirDialog(self, 
                self.dialogLabel if self.dialogLabel else "Choose a directory",
                style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON | wx.DD_DIR_MUST_EXIST,
                defaultPath = self.defaultPath,)
            if dialog.ShowModal() == wx.ID_OK:
                self.Path = dialog.GetPath()
            dialog.Destroy()
            _ParentWindowObject.UnblockRepaint()
            if self.Path is not None:
                self.OnClick(None)
        except:
            import traceback
            traceback.print_exc(500)
        finally:
            _ParentWindowObject.UnblockRepaint()

    def OnClickOpenFile(self, evt):
        global _ParentWindowObject
        _ParentWindowObject = self.Parent
        _ParentWindowObject.BlockRepaint()
        try:
            dialog = wx.FileDialog(self, 
                self.dialogLabel if self.dialogLabel else "Select a file",
                style = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
                defaultDir = os.path.dirname(UnpackParam(self.defaultPath, os.path.expanduser('~'))),
                defaultFile = os.path.basename(UnpackParam(self.defaultPath, os.path.expanduser('~'))),)
            if dialog.ShowModal() == wx.ID_OK:
                self.Path = dialog.GetPath()
            dialog.Destroy()
            _ParentWindowObject.UnblockRepaint()
            if self.Path is not None:
                self.OnClick(None)
        except:
            import traceback
            traceback.print_exc(500)
        finally:
            _ParentWindowObject.UnblockRepaint()

    def OnClickSaveFile(self, evt):
        global _ParentWindowObject
        _ParentWindowObject = self.Parent
        _ParentWindowObject.BlockRepaint()
        try:
            dialog = wx.FileDialog(self, 
                self.dialogLabel if self.dialogLabel else "Select a filename and location",
                style = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
                defaultDir = os.path.dirname(UnpackParam(self.defaultPath, os.path.expanduser('~'))),
                defaultFile = os.path.basename(UnpackParam(self.defaultPath, os.path.expanduser('~'))),
                )
            #dialog.SetFilename(os.path.basename(UnpackParam(self.defaultPath, os.path.expanduser('~'))))
            if dialog.ShowModal() == wx.ID_OK:
                self.Path = dialog.GetPath()
            dialog.Destroy()
            _ParentWindowObject.UnblockRepaint()
            if self.Path is not None:
                self.OnClick(None)
        except:
            import traceback
            traceback.print_exc(500)
        finally:
            _ParentWindowObject.UnblockRepaint()

    def GetValue(self):
        return self.Path


class TextInput(wx.TextCtrl, FormControlMixin):
    __metaclass__ = TypeHandler("TEXT")

    def __init__(self, parent, form, tag, parser, *args, **kwargs):
        style = kwargs.get("style", 0)
        if tag.HasParam("READONLY"):
            style |= wx.TE_READONLY
        if form:
            style |= wx.TE_PROCESS_ENTER
        kwargs["style"] = style
        wx.TextCtrl.__init__(self, parent, *args, **kwargs)
        FormControlMixin.__init__(self, form, tag)
        self.Bind(wx.EVT_TEXT_ENTER, self.OnEnter)
        self.SetValue(GetParam(tag, "VALUE", ''))
        ml = int(GetParam(tag, "MAXLENGTH", 0))
        self.SetMaxLength(ml)
        if ml and len(self.GetValue()) > ml:
            self.SetValue(self.GetValue()[:ml])
        if tag.HasParam("SIZE"):
            size = max(int(tag.GetParam("SIZE")), 5)
            width = self.GetCharWidth() * size
            self.SetSize((width, -1))


class PasswordInput(TextInput):
    __metaclass__ = TypeHandler("PASSWORD")

    def __init__(self, parent, form, tag, parser):
        TextInput.__init__(self, parent, form, tag, parser, style=wx.TE_PASSWORD)


class Checkbox(wx.CheckBox, FormControlMixin):
    __metaclass__ = TypeHandler("CHECKBOX")

    def __init__(self, parent, form, tag, parser, *args, **kwargs):
        wx.CheckBox.__init__(self, parent, *args, **kwargs)
        FormControlMixin.__init__(self, form, tag)
        self.value = GetParam(tag, "VALUE", "1")
        if tag.HasParam("checked"):
            self.SetValue(True)
        #self.SetLabel(self.value)

    def GetValue(self):
        if self.IsChecked():
            return self.value
        else:
            return None


class Radio(wx.RadioButton, FormControlMixin):
    __metaclass__ = TypeHandler("RADIO")

    def __init__(self, parent, form, tag, parser, *args, **kwargs):
        self.value = GetParam(tag, "VALUE", "1")
        fnt = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        if tag.HasParam('fontsize'):
            try:
                fntsz = GetParam(tag, 'fontsize').strip()
                if fntsz.startswith('+'):
                    fnt.SetPointSize(fnt.GetPointSize() + int(fntsz))
                elif fntsz.startswith('-'):
                    fnt.SetPointSize(fnt.GetPointSize() - int(fntsz))
                else:
                    fnt.SetPointSize(int(fntsz))
            except:
                import traceback
                traceback.print_exc(500)
        if tag.HasParam('fontweight'):
            try:
                w = {'bold': wx.FONTWEIGHT_BOLD, 
                     'normal': wx.FONTWEIGHT_NORMAL, 
                     'max': wx.FONTWEIGHT_MAX,
                     'light': wx.FONTWEIGHT_LIGHT}
                fnt.SetWeight(w.get(GetParam(tag, 'fontweight'), wx.FONTWEIGHT_NORMAL))
            except:
                import traceback
                traceback.print_exc(500)
        parentFont = parent.GetFont()
        parent.SetFont(fnt)
        wx.RadioButton.__init__(self, parent, label=self.value, *args, **kwargs)
        FormControlMixin.__init__(self, form, tag)
        if tag.HasParam("checked"):
            self.SetValue(True)
        self.SetLabel(self.value)
        parent.SetFont(parentFont)

    def GetValue(self):
        if wx.RadioButton.GetValue(self):
            return self.value
        else:
            return None


class HiddenControl(wx.EvtHandler, FormControlMixin):
    __metaclass__ = TypeHandler("HIDDEN")

    def __init__(self, parent, form, tag, parser, *args, **kwargs):
        wx.EvtHandler.__init__(self)
        FormControlMixin.__init__(self, form, tag)
        self.value = GetParam(tag, "VALUE", "")
        self.enabled = True

    def GetValue(self):
        return self.value

    def Disable(self):
        self.enabled = False

    def IsEnabled(self):
        return self.enabled


class TextAreaInput(wx.TextCtrl, FormControlMixin):
    __metaclass__ = TypeHandler("TEXTAREA")

    def __init__(self, parent, form, tag, parser, *args, **kwargs):
        style = wx.TE_MULTILINE
        if tag.HasParam("READONLY"):
            style |= wx.TE_READONLY
        wx.TextCtrl.__init__(self, parent, style=style)
        FormControlMixin.__init__(self, form, tag)
        if tag.HasEnding():
            src = parser.GetSource()[tag.GetBeginPos():tag.GetEndPos1()]
        else:
            src = ''
        self.SetFont(wx.SystemSettings.GetFont(wx.SYS_ANSI_FIXED_FONT))
        self.SetValue(src)
        cols = int(GetParam(tag, "COLS", 22))
        width = int(float(self.GetCharWidth()) * cols)
        rows = int(GetParam(tag, "ROWS", 3))
        height = int(float(self.GetCharHeight()) * rows)
        self.SetSize((width, height))






import logging

logger = logging.getLogger( 'cspace.util.eventer' )

class Eventer( object ) :
    def __init__( self ) :
        self.__nextId = 0
        self.__callbacks = {}
        self.__callbackIds = {}

    def register( self, event, callback ) :
        callbackId = self.__nextId
        self.__nextId += 1
        cbList = self.__callbacks.setdefault( event, [] )
        cbList.append( (callback,callbackId) )
        self.__callbackIds[callbackId] = event
        return callbackId

    def remove( self, callbackId ) :
        event = self.__callbackIds.pop( callbackId )
        cbList = self.__callbacks[event]
        for i,info in enumerate(cbList) :
            if info[1] == callbackId :
                del cbList[i]
                if not cbList :
                    del self.__callbacks[event]
                return
        assert False

    def trigger( self, event, *args, **kwargs ) :
        cbList = self.__callbacks.get( event )
        if cbList is None :
            logger.warning( 'no event handler registered for event: %s', event )
            return
        for cb,cbid in cbList[:] :
            if cbid in self.__callbackIds :
                cb( *args, **kwargs )



import logging
from ncrypt0.x509 import X509Error
from ncrypt0.ssl import SSLConnection, SSLError
from ncrypt0.rsa import RSAKey, RSAError
from nitro.async import AsyncOp
from nitro.tcp import tcpConnect
from nitro.ssl import sslConnect, sslAbort, SSLMessageStream
from nitro.bencode import encode, decode, DecodeError
from cspace.network.sslutil import makeClientSSLContext
from cspace.network.location import lookupUser
from cspace.network.routerclient import routerConnect

logger = logging.getLogger( 'cspace.network.connector' )

CACHED_CONNECT_REFRESH = 3

class Dummy : pass

def _authenticateUser( sock, sslContext, remotePublicKey, reactor, callback=None ) :
    def doCancel() :
        connectOp.cancel()
        sslAbort( sslConn )
    def onSSLConnect( err ) :
        global logger
        if err is not None :
            logger.error( 'ssl connect err=%s' % str(err) )
            sslAbort( sslConn )
            op.notify( None )
            return
        try :
            peerCert = sslConn.getPeerCertificate()
            peerKey = RSAKey()
            peerKey.fromPKey_PublicKey(peerCert.get_pubkey())
            CN = peerCert.get_subject().CN
            if CN.count('@'):
                peerName, peerKeyID = CN.split('@')
            else:
                peerName, peerKeyID = CN, ''
        except (SSLError,X509Error,RSAError), e :
            logger.exception( 'ssl connect error' )
            sslAbort( sslConn )
            op.notify( None )
            return
        if peerKey.toDER_PublicKey() != remotePublicKey.toDER_PublicKey() :
            logger.error( 'ssl connect public key mismatch' )
            sslAbort( sslConn )
            op.notify( None )
            return
        # logger.info('onSSLConnect SUCCESSFUL with %s' % str(peerName))
        op.notify( sslConn )
    sslConn = SSLConnection( sslContext, sock )
    sslConn.setConnectState()
    connectOp = sslConnect( sslConn, reactor, onSSLConnect )
    op = AsyncOp( callback, doCancel )
    return op

def _directConnect( sslContext, remotePublicKey, directLocation,
        reactor, callback=None ) :
    def onConnect( connector ) :
        if connector.getError() != 0 :
            op.notify( None )
            return
        authOp = _authenticateUser( connector.getSock(), sslContext,
                remotePublicKey, reactor, op.notify )
        op.setCanceler( authOp.cancel )
    connectOp = tcpConnect( directLocation.addr, reactor, onConnect )
    op = AsyncOp( callback, connectOp.cancel )
    return op

def _routedConnect( sslContext, remotePublicKey, routerLocation,
        reactor, callback=None ) :
    def onConnect( sock ) :
        if sock is None :
            op.notify( None )
            return
        authOp = _authenticateUser( sock, sslContext,
                remotePublicKey, reactor, op.notify )
        op.setCanceler( authOp.cancel )
    connectOp = routerConnect( routerLocation.routerAddr, routerLocation.routerId,
            reactor, onConnect )
    op = AsyncOp( callback, connectOp.cancel )
    return op

def serverCertVerifyCallback(conn, cert, errno, depth, preverify_ok):
    return preverify_ok
    
def _locationConnect( localUserName, localKeyId, localRSAKey, localPublicIP,
        remotePublicKey, remoteLocation, reactor, callback=None ) :
    sslContext = makeClientSSLContext( localUserName, localKeyId, localRSAKey, serverCertVerifyCallback )
    directLocations = remoteLocation.directLocations
    routedLocations = remoteLocation.routedLocations
    remotePublicIP = remoteLocation.publicIP
    maybePublic = False
    for dloc in directLocations :
        if remotePublicIP == dloc.addr[0] :
            maybePublic = True
            break
    samePublic = (localPublicIP == remotePublicIP)
    connOps = {}
    def doCancel() :
        for connOp in connOps.keys() :
            connOp.cancel()
        connOps.clear()
    def onConnect( connOp, sslConn ) :
        del connOps[connOp]
        if sslConn is None :
            if not connOps :
                op.notify( None )
            return
        doCancel()
        op.notify( sslConn )
    def doDirectConnect( dloc ) :
        connOp = _directConnect( sslContext, remotePublicKey,
                dloc, reactor, lambda x : onConnect(connOp,x) )
        connOps[connOp] = 1
    def doRoutedConnect( rloc ) :
        connOp = _routedConnect( sslContext, remotePublicKey,
                rloc, reactor, lambda x : onConnect(connOp,x) )
        connOps[connOp] = 1
    def onTimer() :
        del connOps[timerOp]
        for rloc in routedLocations : doRoutedConnect( rloc )
        if not connOps :
            op.notify( None )
    for dloc in directLocations : doDirectConnect( dloc )
    routedDelay = 0
    if (maybePublic or samePublic) and (len(connOps) > 0) :
        routedDelay = 1
    timerOp = reactor.callLater( routedDelay, onTimer )
    connOps[timerOp] = 1
    op = AsyncOp( callback, doCancel )
    return op

def _userConnect( localUserName, localKeyId, localRSAKey, localPublicIP,
        remotePublicKey, locationCache, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.connectOp :
            obj.connectOp.cancel()
            obj.connectOp = None
        if obj.lookupOp :
            obj.lookupOp.cancel()
            obj.lookupOp = None
        if obj.timerOp :
            obj.timerOp.cancel()
            obj.timerOp = None
    def onError() :
        doCancel()
        op.notify( None )
    def doNotify( sslConn ) :
        doCancel()
        op.notify( sslConn )
    def onConnect( sslConn ) :
        obj.connectOp = None
        if sslConn is not None :
            doNotify( sslConn )
            return
        if not obj.cached :
            onError()
            return
        if obj.timerOp :
            obj.timerOp.cancel()
            obj.timerOp = None
        old = obj.location
        obj.location = locationCache.getLocation( remotePublicKey )
        if old != obj.location :
            if obj.location is None :
                onError()
                return
            if obj.lookupOp :
                obj.lookupOp.cancel()
                obj.lookupOp = None
            obj.cached = False
            doConnectLocation()
            return
        if not obj.lookupOp :
            obj.lookupOp = locationCache.refreshUser( remotePublicKey,
                    onLookup )
    def onTimer() :
        obj.timerOp = None
        assert not obj.lookupOp
        obj.lookupOp = locationCache.refreshUser( remotePublicKey,
                onLookup )
    def onLookup( location ) :
        obj.lookupOp = None
        assert not obj.timerOp
        if location is None :
            onError()
            return
        if obj.location == location :
            if obj.connectOp :
                obj.cached = False
                return
            onError()
            return
        obj.cached = False
        obj.location = location
        if obj.connectOp :
            obj.connectOp.cancel()
            obj.connectOp = None
        doConnectLocation()
    def doConnectLocation() :
        obj.connectOp = _locationConnect( localUserName, localKeyId, localRSAKey,
                localPublicIP, remotePublicKey, obj.location,
                reactor, onConnect )
    obj.cached = False
    obj.lookupOp = None
    obj.connectOp = None
    obj.timerOp = None
    obj.location = locationCache.getLocation( remotePublicKey )
    if obj.location is None :
        obj.lookupOp = locationCache.refreshUser( remotePublicKey,
                onLookup )
    else :
        doConnectLocation()
        timeout = CACHED_CONNECT_REFRESH
        obj.timerOp = reactor.callLater( timeout, onTimer )
        obj.cached = True
    op = AsyncOp( callback, doCancel )
    return op

def _serviceConnect( sslConn, serviceName, reactor, callback=None ) :
    def doCancel() :
        ms.shutdown()
        sslAbort( sslConn )
    def onClose( *args ) :
        doCancel()
        op.notify( None )
    def onInput( data ) :
        try :
            result = decode( data )
            assert type(result) is int
            assert result == 0
        except :
            doCancel()
            op.notify( None )
            return
        ms.shutdown()
        op.notify( sslConn )
    ms = SSLMessageStream( sslConn, reactor )
    ms.setCloseCallback( onClose )
    ms.setErrorCallback( onClose )
    ms.setInvalidMessageCallback( onClose )
    ms.setInputCallback( onInput )
    ms.sendMessage( encode(serviceName) )
    ms.enableRead( True )
    op = AsyncOp( callback, doCancel )
    return op

def userServiceConnect( localUserName, localKeyId, localRSAKey, localPublicIP,
        remotePublicKey, remoteServiceName, locationCache,
        reactor, callback=None ) :
    def onConnect( sslConn ) :
        if sslConn is None :
            op.notify( sslConn )
            return
        serviceOp = _serviceConnect( sslConn, remoteServiceName,
                reactor, op.notify )
        op.setCanceler( serviceOp.cancel )
    connectOp = _userConnect( localUserName, localKeyId, localRSAKey, localPublicIP,
            remotePublicKey, locationCache, reactor, onConnect )
    op = AsyncOp( callback, connectOp.cancel )
    return op
